(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     70861,       1443]*)
(*NotebookOutlinePosition[     71571,       1468]*)
(*  CellTagsIndexPosition[     71527,       1464]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[BoxData[
    \(\( (*\ 
      This\ code\ was\ written\ by\ Alan\ Mason . \ \ It\ is\ intended\ to\ 
        handle\ the\ calculations\ in\ Itzykson\ and\ Zuber, \ 
      pp . \ 419 - 
        422. \ Their\ notation\n\tis\ used\ throughout . \ A\ TEX\ file\ is\ 
          available\ that\ explains\ the\ analysis\ underlying\ this\ 
          \(code . \)\ *) \n
    \(ExpandAlphas\  = 
      \ {\[CapitalSigma]\  -> \ 
          \[Alpha][1] + \[Alpha][2] + \ \[Alpha][3] + \ \[Alpha][4], \n
        \[Alpha][n_Integer]\  :> \ 
          \[Alpha][Floor[n/10]] + \[Alpha][Mod[n, \ 10]]\  /; \ n\  > \ 9}; 
    \)\)\)], "Input",
  CellLabel->"In[1]:="],

Cell[BoxData[
    \(\(CompactAlphas\ \  = 
      \ {\ \ \[Alpha][1] + \[Alpha][2] + \ \[Alpha][3] + \ \[Alpha][4]\  -> \ 
          \[CapitalSigma], \n\t\t\ 
        \(-\[Alpha][1]\) - \[Alpha][2] - \ \[Alpha][3] - \ \[Alpha][4]\  -> \ 
          \(-\[CapitalSigma]\), \n\t\t
        \[Alpha][u_] + \[Alpha][v_]\  :> \ \ 
          \[Alpha][10\ u\  + \ v]\  /; \ 
            \((u\  < \ 10)\)\  && \ \((v\  < \ 10)\), 
        \(-\[Alpha][u_]\) - \[Alpha][v_]\  :> \ \ 
          \(-\[Alpha][10\ u\  + \ v]\)\  /; \ 
            \((u\  < \ 10)\)\  && \ \((v\  < \ 10)\)\n\t}; \)\)], "Input",
  CellLabel->"In[2]:="],

Cell[BoxData[
    \(\(ea\  := \ ExpandAlphas; \ ca\  := \ CompactAlphas; \t\t\)\)], "Input",\

  CellLabel->"In[3]:="],

Cell[BoxData[
    \(\( (*\ 
      The\ following\ recursively\ computes\ the\ trace\ of\ Dirac\ gamma\ 
        matrices\ \((of\ Clifford\n\t\t\t\ algebra)\)\ in\ symbolic\ 
        \(form\ --\)\ i . e . , \ 
      as\ an\ explicit\ product\ of\ pairwise\ delta - functions\ *) \n
    Tr[L_List, \ d_:  4]\  := \ 
      Module[{ans = 0\ , i, j, \ len, \ L1 = L, \ L2, \ u, \ v}, \n\t\t
        len\  = \ Length[L1]; \n\t\t\tIf[OddQ[len], \ Return[0], \ 1]; \ \n
        \t\t\t\tIf[len\  == \ 0, \ Return[d], \ 1]; \n\t\t
        If[len\  == \ 2, \ 
          Return[\(-d\)\ \[Delta][L1[\([1]\)], \ L1[\([2]\)]]], 1]; \n\t\t
        i\  = \ L1[\([1]\)]; \n\t\t\t\ L1\  = \ Rest[L1]; \n\t\t\t\t
        For[j = 1, \ j <= \ len\  - \ 1, \ \(j++\), \ \ L2 = Delete[L1, \ j]; 
          \n\t\t\t\t\t\t
          ans\  = \ 
            ans\  + \ 
              \[Delta][i, \ L1[\([j]\)]]\ \ \((\(-1\))\)^j\ Tr[L2, \ d]]; 
        \  (*\ Tr[L1\  = \ Delete[L1, \ j], \ d]\ gives\ a\ bug\ *) \n\t\t\t
        ans\  = \ SimplifyDeltas[ans]\n\t\t\t\t]\)\)], "Input",
  CellLabel->"In[4]:="],

Cell[BoxData[
    \(Intd[u_ + v_, \ x_, \ dim_, \ vectors_]\  := \ 
      Intd[u, \ x, \ dim, \ vectors] + Intd[v, \ x, \ dim, \ vectors]\)], 
  "Input",
  CellLabel->"In[5]:="],

Cell[BoxData[
    \(\(Intd[\ Exp[g_]\ f_, \ x_, \ dim_, \ vectors_]\  := \ 
      Module[{\ a, \ b, \ c, d = \ dim, \ ans, f1, \ g1, Lindices, \ Lterms, 
          len, \ u, vecs\  = \ vectors, \ VR\ }, \n\t\tg1\  = \ Expand[g]; \n
        \t\ta\  = \ Coefficient[g1, \ x^2]; \n\t\t
        b\  = \ Coefficient[g1, \ x]; \n\t\tc\  = \ g1 //. \ x -> \ 0; \n
        \t (*\tans\  = \ Exp[b^2/\((4\ a)\)]/a^\((d/2)\); \ *) \n\t\t
        f1\  = \ TuckIndices[f\ , \ vecs]; \n\t\tf1\  = \ Expand[f1, \ x]; 
        \n (*\t\tLterms\  = \ List\ @@\ f1; \ *) \n\t\t
        f1\  = \ f1\  /. \ 
            x[u_]\  :> \ 
              x[u]/\ \((\(-a\))\)^\((1/2)\)\  + \ 
                b[u]/\((2\ \((\(-\ a\))\))\); \  (*\ NOT\  //. \ *) \n\t\t
        f1\  = \ TuckIndices[f1\ , \ vecs]; \n\t\t\t\n\t\t
        f1\  = \ Expand[f1]; \n\t\t\tVR\  = \ WriteVectorRules[vecs]; \n
        \t\t\t\tf1\  = \ f1\  //. \ VR; 
        \n (*\ f1\  = \ Expand[f1]; \ *) \n (*\ Lterms\  = \ List\ @@\ f1; 
          \ \ CAUSES\ \(BUG!!\)\ If\ there' s\ just\ one\ term, \ 
          Expand\ gives\ an\ expression\ with\ head\ Times, \ 
          not\ \(Plus!\)\ *) \n\t\t
        If[Head[f1] === \ Plus, \ Lterms\  = \ List\ @@\ f1, \ 
          Lterms\  = \ f1]; \n\t\t
        If[\ Head[Lterms]\  =!= \ List, \ Lterms\  = \ List[Lterms]]; \n\t\t
        Lterms\  = \((\ Intd2[#, x, \ d]&\ )\)/@\ Lterms; \n\ \t\t\t\t\t
        ans\  = \ Plus\ @@\ Lterms; \nans\  = \ SimplifyDeltas[ans]; \n
        \t\t\t\t\t\t
        ans\  = \ 
          ans\ \ Exp[c + b^2/\((4 \((\(-\ a\))\))\)]/\((\(-a\))\)^\((d/2)\)\n
        \t]; \)\)], "Input",
  CellLabel->"In[6]:="],

Cell[BoxData[
    \(\(\tIntd2[term_, \ x_, \ dim_]\  := \ 
      Module[{\ \[Alpha], \ \[Beta], \[Mu], \ \[Nu], ans, \ \ c0 = 0, \ 
          c2 = 0, \ c4 = 0, d\  = \ dim, fac, f1\  = \ term, \ len, \ 
          Lindices}, \n\t\tfac\  = \ \((2\ Pi)\)^d/\((4\ Pi\ )\)^\((d/2)\); \t
        \n\t\tLindices\  = \ GetIndices[f1, \ x]; \n\t\t
        len\  = \ Length[Lindices]; \n\t\t
        If[len\  > \ 0, \ \[Mu]\  = \ Lindices[\([1]\)]]; \n\t\t
        If[len\  > 1, \[Nu]\  = \ Lindices[\([2]\)]\ ]; \n\t\t\ 
        If\ [len\  > \ 2, \ \[Alpha]\  = \ Lindices[\([3]\)]\ \ ]; \n\t\t\t\t
        If[len\  > 3, \ \ \[Beta]\  = \ Lindices[\([4]\)]\ \ ]; \n\t\t
        c0\  = \ f1\  //. \ x[u_]\  -> \ 0; \n\t\t\t
        If[1\  < \ len\  < \ 4, \ 
          c2\  = Coefficient[f1, \ x[\[Mu]]\ x[\[Nu]]]\ ]; \n\t\t\t
        If[len > 3, \ \t\t
          c4\  = \ Coefficient[f1, \ 
              x[\[Mu]]\ x[\[Nu]]\ x[\[Alpha]]\ x[\[Beta]]]\ \ ]; \n\t\t\t\t
        ans\  = fac 
            \((\ c0\  + \ c2\ \[Delta][\[Mu], \ \[Nu]]/2\t + 
                c4\ \((\[Delta][\[Mu], \[Nu]]\ \[Delta][\[Alpha], \[Beta]] + 
                        \[Delta][\[Mu], \[Alpha]]\ \[Delta][\[Nu], \[Beta]] + 
                        \[Delta][\[Mu], \[Beta]]\ \[Delta][\[Nu], \[Alpha]])
                      \)/4)\); \n\t\tans\  = \ ans\  //. \ x[_] -> \ 0\ ; \n
        \t\tReturn[ans]; \n\t]\)\)], "Input",
  CellLabel->"In[7]:="],

Cell[BoxData[
    \(\(GetIndices[term_, \ x_]\  := \ 
      Module[{L, t = term, \ u}, \n\tIf[\ FreeQ[term, \ x[_]], \ Return[{}]]; 
        \n\t\tt\  = \ t/\((term\  /. \ x[u_]\  :> \ 1)\); \t\n\t
        L\  = \ t\  /. \ Times -> \ List; \t\n
        L\  = \ L\  //. \ Power[x[u_], \ n_]\  :> \ Array[u, \ n]; \n\t\t
        L\  = \ L\  //. \ x[u_]\  -> \ {u}; \nL\  = \ Flatten[L]; \n
        If[\ Length[L]\  > \ 1, L\  = \ L\  /. \ u_[_]\  :> \ u, \ 
          L\  = \ \ L\  /. \ List[u_[_]]\  -> \ List[u]]\n\t]; \)\)], "Input",\

  CellLabel->"In[8]:="],

Cell[BoxData[
    \(\(WriteVectorRules[Vectors_List]\  := \ 
      Module[{i, \ len, L = Vectors, \ u, v, \ w, \ VR = {}}, \n\t
        len\  = \ Length[L]; \n\t
        For[i = 1, \ i\  <= \ len, \ \(i++\), \n\t\t\t
          VR\  = \ Append[VR, 
              \ {\((u___\ \ L[\([i]\)]\ \ v___)\)[w_]\  -> \ 
                  u\ \ \(L[\([i]\)]\)\ [w]\ v}]\n\t\t\t\ \ ]; \n\t
        Flatten[VR]\n\t]; \)\)], "Input",
  CellLabel->"In[9]:="],

Cell[BoxData[
    \(\(TuckIndices[e_, \ Vectors_List]\  := \ 
      Module[{a, \ ans, \ b, \ i, \ \ u, \ v, \ w, \ rules, \ 
          vecs\  = \ Vectors}, \n\t
        rules\  = 
          \ {\((u_ + v_)\)[w_] -> u[w] + v[w], 
            \((a_\ u_)\)[w_] :> a\ \ u[w] /; FreeAllQ[a, vecs]\ \  (*\ , \n
              \t\t\t\t\ u_[v_] \[RuleDelayed] \(Expand[u]\)[v]\ *) }; \n\t
        ans\  = \ \ e\  //. \ rules; \n\t\tans\  /. \ 0[_]\  -> \ 0\n\t]; 
    \)\)], "Input",
  CellLabel->"In[10]:="],

Cell[BoxData[
    \(FreeAllQ[e_, \ L_List]\  := \ 
      Module[{L1}, \nL1\  = \ \((FreeQ[e, \ #]&)\)\ /@\ L; \n
        If[MemberQ[L1, \ False], \ \t\tFalse, \ True]]\)], "Input",
  CellLabel->"In[11]:="],

Cell[BoxData[
    \(\(SimplifyDeltas[e_]\  := \ 
      Module[{ans\  = \ e}, \n\t\tSetAttributes[\[Delta], \ Orderless]; \n
        \t\ \ \tans\  = \ ans\  //. \t\ \[Delta][u_, \ u_]\  -> \ 1; \n\t\t
        ans\  = \ ans\  //. \ \[Delta][u__]^v_\  -> \ \[Delta][u]; \n\t\t\ 
        ans\  = \ 
          ans\  //. \ \ \[Delta][u_, \ v_]\  :> \ \ 0\  /; \ u\  != \ v\n\t\t]
        \t\t\ \ \t\t\)\)], "Input",
  CellLabel->"In[12]:="],

Cell[BoxData[
    \(CompleteTheSquareFull[e_, \ x_]\  := \ \n
      Module[\ {a, \ b, \ \ e0, \ coef}, \n\t\t\t\t\t\ e0\  = \ Expand[e]; \n
        \t\t\t\t\t\ta\  = \ Coefficient[e0, \ x, \ 2]; \n\t\t\t\ 
        b\  = \ Coefficient[e0, \ x, \ 1\ ]; \n\t\t\t\t
        e0\  = \ Collect[e0, \ x] //. \ \ x\ \ b\ \  -> \ 0\ ; \n\t\t\t
        e0\  = \ e0\  - \ b^2\ \ /\((4\ a)\); \n\t\t
        e0\  = \ Collect[e0, \ x^2]; \n\t\t\t
        e0\  = \ e0 /. \ x^2 -> Hold[\ \((x\  + \ b/\((2\ a)\))\)^2\ ]; \ \n
        e0\  = \ Expand[e0]; \n\t\te0\  = \ Collect[e0, \ Hold[_]]; \n\t\t\t
        e0\  = \ ReleaseHold[e0]\ \ ]\)], "Input",
  CellLabel->"In[13]:="],

Cell[BoxData[
    \(ctsf\  := \ CompleteTheSquareFull\)], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[{
    \(GetFree[u_\ v_, \ vars_List]\  := \ 
      GetFree[u, \ vars]\ GetFree[v, \ vars]\), 
    \(GetFree[term_, \ vars_List]\  := \ 
      If[\ FreeAllQ[term, \ vars], \ term, \ 1]\)}], "Input",
  CellLabel->"In[15]:="],

Cell[BoxData[{
    \(GetSummationIndicesAndRanges[term_]\  := \n\t
      Module[{e = term, \ i, \ indices, \ L}, \n\t\t\t
        If[FreeQ[term, \ Sum], \ Return[{}]]; \n\t\t\t\t\t
        L\  = \ List\ @@\ e; 
        \  (*\ e\  = \ Sum[func, \ {var, \ lower, \ upper}, \  ... \ ]\ *) \n
        \t\tL\  = \ Rest[L]\ \n\t\t\t]\), 
    \(\(gsir\  := \ GetSummationIndicesAndRanges; \)\)}], "Input",
  CellLabel->"In[17]:="],

Cell[BoxData[
    \(gsi\  := \ GetSummationIndices; \n
    GetSummationIndices[term_]\  := \ 
      Module[{e\  = \ term, \ i, \ indices, \ L\ }, \n\t\t
        If[FreeQ[Head[term], \ Sum], \ Return[{}]]; \n\t\t\t
        L\  = \ List\ @@\ e; 
        \  (*\ Sum[func, \ {var, \ lower, \ upper}, \  ... \ ]\ *) \n\t\t
        L\  = \ Rest[L]; 
        \ \  (*\ L\ is\ {{var, \ lower, \ upper}, \  ... \ }\ *) \n\t\t
        \(\((\ # /. \ # -> #[\([1]\)]\ )\)&\)\ /@\ L\n\t]\)], "Input",
  CellLabel->"In[19]:="],

Cell[BoxData[{
    \(gsr\  := \ GetSummationRanges\), 
    \(GetSummationRanges[term_]\  := \ 
      Module[{e\  = \ term, \ i, \ indices, \ L\ }, \n\t\t
        If[FreeQ[Head[term], \ Sum], \ Return[{}]]; \n\t\t\t
        L\  = \ List\ @@\ e; 
        \  (*\ Sum[func, \ {var, \ lower, \ upper}, \  ... \ ]\ *) \n\t\t
        L\  = \ Rest[L]; 
        \ \  (*\ L\ is\ {{var, \ lower, \ upper}, \  ... \ }\ *) \n\t\t
        \(\((\ # /. \ # -> Rest[#])\)&\)\ /@\ L]\)}], "Input",
  CellLabel->"In[20]:="],

Cell[BoxData[{
    \(GetSummationFunction[term_]\  := \ 
      Module[{e\  = \ term, \ i, \ indices, \ L\ }, \n\t\t\t
        If[FreeQ[Head[term], \ Sum], \ Return[{}]]; \n\t\t\t
        L\  = \ List\ @@\ e; 
        \  (*\ e\  = \ \ Sum[func, \ {var, \ lower, \ upper}, \  ... \ ]\ *) 
          \n\ \ L[\([1]\)]\n\t]\n\t\t\t\t\t\), 
    \(gsf\  := \ GetSummationFunction\)}], "Input",
  CellLabel->"In[22]:="],

Cell[BoxData[{
    \(ss\  := \ SimplifySum\), 
    \( (*\ OMIT\ \(?? \ \(--\ causes\)\ too\ much\ recursion\)\ *) \ \ 
    SimplifySum[u_ + v_]\  := \ SimplifySum[u] + SimplifySum[v]; \n
    SimplifySum[u_\ v_]\  := \ u\ \ SimplifySum[v]\  /; \ FreeQ[u, \ Sum]; \n
    \ SimplifySum[\(Hold[Sum]\)[f_ + g_, \ {v__}]]\  := \ 
      SimplifySum[\(Hold[Sum]\)[f, \ {v}]]\  + \ 
        SimplifySum[\(Hold[Sum]\)[g, \ {v}]]; \ 
    \n (*\ This\ last\ rule\ is\ optional\ *) \t\t\t\t\)}], "Input",
  CellLabel->"In[24]:="],

Cell[BoxData[
    \(SimplifySum[term_]\  := \ 
      Module[{\ ans, PositionsToDelete\  = \ {}, \ e = term, func, \ 
          funcFreePart = 1, \ funcProtected, \ \ i, L, \ L1, Lfinal, \ len, \ 
          lower, \ \ SummationData, \ SummationDataFinal, \ 
          SummationIndicesUnique\ , \ upper, \ var, \ XXT\ \ }, \n
        \t (*\tPrint[\ "\<term is \>", \ term]; \ \ *) \n\t
        If[FreeQ[Head[e], \ Sum], \ Return[e]]; \ \n\t\t
        SummationData\  = \ GetSummationIndicesAndRanges[e]; \n\t\t
        SummationDataFinal\  = \ SummationData; \n\t\t\t\t
        func\  = \ GetSummationFunction[e]; \n
        \t\t (*\ For\ greatest\ generality, \ 
          we\ may\ want\ to\ force\ Sum[f_ + g_, \ {indices}]\  = \ 
            Sum[f_, \ {indices}]\  + \ Sum[g_, \ {indices}]\ *) \n
        \t\t\t\t\t\t (*\ \ Now\ protect\ the\ heads, \ 
          in\ case\ a\ summation\ index\ \[Alpha]\ also\ appears\ as\ a\ 
            \(head : \ \[Alpha][x, \ y]\)\ *) \t\n\t\t\t\t\t
        SummationIndicesUnique\  = \ 
          \((Unique[#[\([1]\)]]&)\)\ /@\ SummationData; \n\t\t
        funcProtected\  = \ func; \n\t\tlen\  = \ Length[SummationData]; \n
        \t\t\ For[i = 1, \ i <= \ len, \ \(i++\), \ 
            index\  = \ SummationData[\([i, \ 1]\)]; 
            funcProtected\ \  = \ 
              funcProtected\  /. \ \ \ 
                index[u__]\  :> \(SummationIndicesUnique[\([i]\)]\)[u]; ]\n
          \t\t\tIf[Head[func]\  === \ Times, \ 
            funcFreePart\  = \ 
              GetFree[funcProtected, \ \((#[\([1]\)]&)\)\ /@\ SummationData]; 
            \n\t\t\ \ \ \ \ \ \ \ \ \ 
            funcProtected\  = \ funcProtected\  /. \ funcFreePart\  -> \ 1]; 
        \n\t\t\tFor[i = 1, \ i <= \ len, \ \(i++\), \n
          \t\t\t\t\ \ \ \ \ \ \ \ \ index\  = \ SummationData[\([i, \ 1]\)]; 
          \n\t\t\t\t\ \ \ \ \ \ \ \ \ \ 
          lower\  = \ SummationData[\([i, \ 2]\)]; \n
          \t\t\t\t\ \ \ \ \ \ \ \ \ \ 
          upper\  = \ \ SummationData[\([i, \ 3]\)]; \ \ \n
          \t\t\t\t\ \ \ \ \ \ \ \ \ \ \ 
          If[\ FreeQ[funcProtected, \ index], \n\ \ \ \ \ \ \ \ \t\t\t\t\ \ 
            funcFreePart\  = \ funcFreePart\ \((upper - lower + 1)\); \n
            \t\t\t\t\ \ 
            PositionsToDelete\  = \ 
              Flatten[List[PositionsToDelete, \ i\ ]\ \ ]; \n\t\t\t\t\t]; \n
          \t\t]; \n\t\t
        SummationDataFinal\  = \ 
          DropPositions[SummationData, \ PositionsToDelete]; \n\t\t\t\t
        \(\(\(\(\(ans\  = \t\ 
          funcFreePart\ \ 
            XXT[funcProtected, \ Sequence\ @@\ SummationDataFinal]\ \ ; \)\n
        \t\t\t (*\ NOTE : \ If\ replaced\ XXT\ by\ Sum\ here, \ 
          we\ would\ have\ to\ Evaluate\ the\ arguments, \ 
          as\ Sum\ Holds\ its\ arguments\ *) \ \n
        \t\t\t\t\t\t (*\ Now\ unprotect\ *) ; \); \); \); \); \n\ \ 
        len\  = \ Length[SummationIndices]; \ \n\t\t
        For[i = 1, \ i\  <= \ len, \ \(i++\), \n\t\t\t\t\ \ \ \ \ \ 
          \(ans\  = \ 
            ans\  //. \ \ 
              \(SummationIndicesUnique[\([i]\)]\)[u__] -> \ 
                \(\((\((#[\([1]\)]&)\)\ /@\ SummationData)\)[\([i]\)]\)[u]; 
          \)\n\t\ \ \ \ ]; \n\t\t\tlen\  = \ Length[SummationDataFinal]; \n
        \t\t\tIf[len\  == \ 0, \ Return[ans\ \  /. \ XXT[u__]\  -> \ u]]; \ \n
        \t\tans\  = \ ans\  /. \ XXT[u__]\  -> \(Hold[Sum]\)[\ u]\n
        \t (*\tans\  = \ ReleaseHold[ans]\ \ \ *) \n\t\t\t]\)], "Input",
  CellLabel->"In[26]:="],

Cell[BoxData[
    \(\( (*\ It' s\ important\ to\ avoid\ recursion\ in\ the\ following\ *) \n
    EinsteinSummationConventionWithDeltasTerm[\ e_, \ SummationIndices_List, 
        \ symbol_:  \[Delta], \ range_: {1, \ 4}]\  := \n\t\ 
      Module[{\ ans, term = e\ , \ L\  = \ SummationIndices, L2, \ len, \ h, 
          i, \ pos, \ sym = symbol, SumIt, \ ran\  = \ range, \ u, \ v, \ w}, 
        \n\t\t\t\t\t\tterm\  = \ SimplifyDeltas[term]; \n\t\t\ \ \ \ \ \ \ \ 
        L2\  = \ GetRepeatedIndices[term]; \n\t\t
        L\  = \ Intersection[L, \ L2]; \n\t\t\t\t\t\ len\  = \ Length[L]; 
        \n (*\tIf[FreeAllQ[term, \ L], \ Return[term]]; \ NO\ *) \n\t\t
        For[i = len, \ i > 0, \ \(i--\), \ \n\t\t\t\t\t\t
          term\  = \ term\  //. \ sym[u_, \ u_]\  -> \ 1; \n\t\t\t\ \ \ \ \ 
          pos\  = \ \ 
            Position[term, \ \ \ 
              sym[L[\([i]\)], \ u__]\  | \ sym[u__, \ L[\([i]\)]], {0, \ 
                Infinity}, \ 1]; \n
          \t\t\t (*\ 
            BUG\ \(Alert!\)\ If\ fail\ to\ specify\ {0, \ Infinity}\ and\ 
              instead\ use\ just\ Infinity, \ 
            pos\ will\ be\ empty\ on\ things\ like\ 
                \[Delta][\[Alpha], \ \[Beta]] . \ \ 
                  \(Example : \n\t\t\t\t\t\t\ti\)\  = \ 1, \ 
            L[\([i]\)]\  = \ \[Alpha], \ 
            term\  = \ 
              \(\[Delta][\[Alpha], \ \[Beta]] . \ \ pos\ is\ {}\ because\ sym
                  \  = \ \[Delta]\ appears\ as\ the\ head\ of\ \(term!\)\)\ \n
            \t\t\t*) \ \n\t\t\t\t\t\t\n\t\t\t\t\ \ 
          If[\ Length[pos] > \ 0, \ \n\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
            h\  = \(Extract[\ term, \ pos]\)[\([1]\)]; \n
            \t\t\t\t\t\t\ \ \ \ \ \ \ \ \ \ \ \t
            h\  = \ h\ \  //. \ \ 
                sym[L[\([i]\)], \ u__]\  | \ 
                    sym[u__, \ L[\([i]\)]]\ \ \ \  -> \ u; \n
            \t\t\t\t\t\t\t\t\t
            term\  = \ 
              term\ \  //. \ 
                v_[u___, \ L[\([i]\)], \ w___]\  -> \ v[u, \ h, \ w]; \n
            \t\t\t\t\t\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ L\  = \ Drop[L, \ {i}]; \n
            \t\t\t\ \ \ \ \t]; \ \ \  (*\ If\ *) \n\t\t\t\t]; 
        \ \  (*\ For\ *) \n\t\ \ \ \t\t\t\t
        term\  = \ term\  //. \ sym[u_, \ u_]\  -> \ 1; 
        \  (*\ Need\ this\ once\ again, \ 
          to\ replace\ e . g . \ \[Delta][\[Beta], \ \[Beta]]\ by\ 1\n
              \t\t\t\t\tthe\ last\ time\ \(around\ --\)\ when\ i\  = \ 1, \ 
          L[\([i]\)]\  = \ \[Alpha]\ in\ the\ above\ example\ *) \n\t\t
        L2\  = \ \(Flatten[\ Append[{#}, \ ran]]&\)\ /@\ L; \n\t\t
        If[Length[L2]\  == \ 0, \ 
          Return[term\  //. \ Power[sym[u__], \ v_] -> \ sym[u]]]; \n
        \t\t\t\t\ \t\t
        ans\  = \t\t
          SumIt[term\  //. \ Power[sym[u__], \ v_] -> \ sym[u], \ 
            Sequence\ @@\ L2\ ]; \n\t
        ans\  = \ ans\  //. \ SumIt -> \ Hold[Sum]\n\t]\)\)], "Input",
  CellLabel->"In[27]:="],

Cell[BoxData[
    \(\(EinsteinSummationConventionWithDeltas[\ e_, \ SummationIndices_List, 
        \ symbol_:  \[Delta], \ range_: {1, \ 4}]\  := \n\t\t\ 
      Module[{ans, exp, i, len, \ L\  = \ SummationIndices, L1, \ Lunique}, \n
        \t\t\texp\  = \ Expand[e]; \n\t\t
        If[Head[exp]\  === \ Plus, \ L1\  = \ List\ @@\ exp, \ 
          L1\  = \ List[exp]]; \t\ \n\t\tlen\  = \ Length[L]; \n\t\t\t\t
        Lunique\  = \ \((Unique[#]&)\)\ /@\ L; \n\t\t\ 
        For[i = 1, \ i\  <= \ len, \ \(i++\), \n\t\t\t\t\ \ \ \ \ \ 
          exp\  = \ 
            exp\  //. \ \ 
              \(L[\([i]\)]\)[u___]\  :> \ \(Lunique[\([i]\)]\)\ [u]\n
          \t\t\t\t]; 
        \t\t (*\ Protect\ things\ like\ \n\t\t\t\t\t\t\t\t\t\[Alpha][1, \ 2]\ 
            from\ summation\ when\ \[Alpha]\ is\ a\ summation\ 
            \(index . \)\ \ *) \n\t\t\t\t
        ans\  = \ 
          \((EinsteinSummationConventionWithDeltasTerm[#, L, \ symbol, \ 
                  range]&)\)\ /@\ L1; \n\t\t
        If[Length[ans]\  > \ 1, \ ans\  = \ Plus\ @@\ ans, \ 
          ans\  = \ ans[\([1]\)]]; \n\t\t
        For[i = 1, \ i\  <= \ len, \ \(i++\), \n\t\t\t\t\ \ \ \ \ \ 
          \(ans\  = \ 
            ans\  //. \ \ 
              \(Lunique[\([i]\)]\)[u___] -> \ \(L[\([i]\)]\)\ [u]; \)\n
          \t\ \ \ \ ]; \ ans\n
        \t\t (*\ You\ will\ often\ want\ ReleaseHold[SimplifySum[ans]]\ *) \n
        \t]; \)\)], "Input",
  CellLabel->"In[28]:="],

Cell[BoxData[{
    \( (*\ 
      Use\ the\ following\ to\ simplify\ TR\ to\ get\ the\ expression\ in\ 
        IZ, \ p . \ 419. \ Syntax\ 
        \(is : \ \ 
          FullSimplifyTheTrace[TR, 
            \ {\[Nu], \ \[Sigma], \ \[Mu], \ \[Alpha], \ \[Rho], \ \[Beta]}]
            \), \ where\ \[Lambda]\ has\ been\ removed\ from\ the\ list\ *) \n
    \t\ FullSimplifyTheTrace[e_, \ L_]\  := \ 
      Module[{ans = 0, c, \ \ exp = e, \ i, \ j, len\  = \ Length[L], \ L1, \ 
          x, \ y}, \n\t\t\ \ \ L1\  = \ Sort[L]; \n\t\t\ \ \ 
        For[i = 1, \ i\  <= \ len, \ \(i++\), \n\t\t\t\ \ \ \ \ \ \ \ \ \ \ \ 
          \(For[j\  = \ i + 1, \ j\  <= \ len, \ \(j++\), \n
            \t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
            x = L1[\([i]\)]; \ y\  = \ L1[\([j]\)]; \ \ \n
            \t\t\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
            c\  = \ Coefficient[exp, \ \[Delta][x, \ y]]; \n
            \t\t\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
            ans\  = \ ans\  + \ Simplify[c]\ \ \[Delta][x, \ y]; \n
            \t\t\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
            exp\  = \ Simplify[exp\  - \ \ c\ \[Delta][x, \ y]]; \n
            \t\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
            If[exp\  == \ 0, \ Return[ans]]; \n
            \t\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ ]; \)\n
          \t\t\ \ \ \ \ \ \ \ ]; \n\t\ \ \ \ ans]\), 
    \(\t\(dotorder\  = \ \ 
      HoldPattern[Dot[u__]\ ] :> Dot[\ Sequence\ @@\ Sort[{u}]]; \)\)}], 
  "Input",
  CellLabel->"In[29]:="],

Cell[BoxData[{
    \(\ ss\  := \ SimplifySum\), 
    \(sd\  := \ SimplifyDeltas; \nhscd\  := \ HoldSummationConventionDeltas; 
    \n\t\t\n (*\ 
      Use\ HoldSummationConventionDeltas\ followed\ by\ SimplifySum\ to\ feed
        \ ToDot\ *) \nToDot[e_\  + \ f_]\  := ToDot[e]\  + \ ToDot[f]\), 
    \(ToDot[e_\ f_]\  := \ e\ ToDot[f]\  /; \ FreeQ[e, \ Sum]\), 
    \(ToDot[1]\  = \ 1; \nToDot[0]\  = \ 0; \)}], "Input",
  CellLabel->"In[31]:="],

Cell[BoxData[
    \(\(ToDot[e_]\  := \ 
      Module[{func, \ i, DropList = {}, IndexAndRangeList, \ L, \ \ len, \ 
          lowerLimit, \ upperLimit, \ tsum, \ \ var, \ \ weight\ }, \ \n
        \ \ \ \ \ tsum\  = \ e; \n\t\t
        IndexAndRangeList\  = \ GetSummationIndicesAndRanges[tsum]; \n\t\t\t\t
        len\  = \ Length[IndexAndRangeList]; \n\t\t\t
        If[len\  == \ 0, \ Return[e]]; \n\ \t\t
        func\  = \ GetSummationFunction[tsum]; \n\t\t\t\t
        For[i = 1, \ i <= \ len, \ \(i++\), \n\t\t
          var\  = \ IndexAndRangeList[\([i, \ 1]\)]; \n\t\t
          lowerLimit = IndexAndRangeList[\([i, \ 2]\)]; \t\t\t\n\t\t\t
          upperLimit\  = \ IndexAndRangeList[\([i, \ 3]\)]; \ \ \n
          \t\t\t (*\ Recall : \ Sum[func, \ {i, \ il, \ iu}], \n\t\t\t\t
            var\  = \ i, \ lowerLimit\  = \ i1, \ upperLimit\  = \ iu\ *) \n
          \t\t\t\tweight\  = \ CountWithMultiplicity[func, \ var]; \n\t\t\t\t
          If[weight == 0, \ func = func\ \((upperLimit - lowerLimit + 1)\); \n
            \t\t\t\t\t\t\t
            DropList\  = \ Append[DropList, \ IndexAndRangeList[\([i]\)]]; \n
            \t\t\t\ \ \ \ \ \ \ \ Continue[]\n\t\t\t]; \n\t\t\t
          If[weight == 2, \ \n\t\t\t\ \ \ \ 
            func\  = \ func\  /. \ u_[var]\ x_[var] -> \ u . x; \n\t\t\t\t\ \ 
            func\  = \ func\  /. \ u_[var]^2 -> \ u . u; \n\t\t\t\t\t
            DropList\  = \ Append[DropList, IndexAndRangeList[\([i]\)]]; \n
            \t\t\t\t\tContinue[]; \n\t\t\t\t]\n\t\t\t]\ ;  (*\ For\ *) \n
        \t\t\t\t\t\t\ L\  = \ \t\tComplement[IndexAndRangeList, \ DropList]; 
        \n\t\t\t\t\t\t\tIf[Length[L]\  == 0, \ Return[func]]; \n\t\t\t
        ss[\t\ Sum[Evaluate[func], Evaluate[\ Sequence@@L]]]\n\t\t\t\t\t]\n
    \)\)], "Input",
  CellLabel->"In[35]:="],

Cell[BoxData[
    \(ExpandDotsTerm[e_, \ vecs_List]\  := \ 
      Module[{dotorder, \ exp\  = \ ExpandAll[e], \ L\  = \ vecs, 
          len\  = \ Length[L], rules}, \n\t\t
        If[Head[exp]\  =!= \ \ Dot, \ Return[exp]]; \n\t\t\t
        rules\  = 
          \ {\(Verbatim[Dot]\)[\ u_\  + \ w_, \ v_] :> \ \ 
              Dot[u, \ v]\  + \ Dot[w, \ v], 
            \(Verbatim[Dot]\)[\ u_\ , \ v_\  + \ w_] :> \ \ 
              Dot[u, \ v]\  + \ Dot[u, \ w], 
            \(Verbatim[Dot]\)[\ u_\  - \ w_, \ v_] :> \ \ 
              Dot[u, \ v]\  - \ Dot[w, \ v], 
            \(Verbatim[Dot]\)[\ u_\ , \ v_\  - \ w_] :> \ \ 
              Dot[u, \ v]\  - \ Dot[u, \ w]}; \n\t\t\t
        exp\  = \ exp\  //. \ rules; \n\t\t\t
        rules\  = 
          \ {\(Verbatim[Dot]\)[a_\ \ u_, \ v_] :> \ 
              a\ Dot[u, \ v]\  /; \ FreeAllQ[a, \ L], \n\t\t\t\t\ \ 
            \(Verbatim[Dot]\)[\ u_, \ a_\ \ v_] :> \ 
              a\ Dot[u, \ v]\  /; \ FreeAllQ[a, \ L]}; \n\t\t
        exp\  = \ exp\  //. \ rules; \n\t\t
        dotorder\  = 
          \ \ {HoldPattern[Dot[u__]\ ] :> Dot[\ Sequence\ @@\ Sort[{u}]]}; \n
        \t\texp\  = \ exp\  //. \ dotorder]\)], "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[
    \(\(ExpandDots[e_, \ vecs_List]\  := \ 
      Module[{exp\  = ExpandAll[\ e], \ L}, \n\t\t
        If[Head[exp]\  === \ Plus, \ L\  = \ List\ @@\ exp, \ 
          L\  = \ List[exp]]; \n\t\ \ \ 
        L\  = \ L\  //. \ 
            \(Verbatim[Dot]\)[u_, \ v_]\  :> \ 
              ExpandDotsTerm[Dot[u, \ v], \ vecs]; \n\t
        L\  = Expand[\ Simplify[L]]; \n\tPlus\ @@\ L]\t\t\)\)], "Input",
  CellLabel->"In[37]:="],

Cell[BoxData[
    \(ReplaceDots[e_, \ vecs_List, \ dim_]\  := \ 
      Module[{exp\  = \ ExpandAll[e], L\  = \ vecs}, \ \t
        If[Head[exp]\  === \ Plus, \ L\  = \ List\ @@\ exp, \ 
          L\  = \ List[exp]]; \n\t\ \ \ 
        L\  = \ L\  //. 
            \(Verbatim[Dot]\)[u_, \ v_]\  :> \ 
              ReplaceDotsTerm[Dot[u, \ v], \ vecs, \ dim]; \n\t\t\ \ \ \ \ 
        L\  = \ \ Expand[\ Simplify[L]]; \n\tPlus\ @@\ L]\)], "Input",
  CellLabel->"In[38]:="],

Cell[BoxData[
    \(\(ReplaceDotsTerm[e_, \ vecs_List, \ dim_]\  := \ 
      Module[{dotorder, \ exp\  = e, i, \ \ L\  = \ vecs, 
          len\  = \ Length[L], rules}, \n\t\t
        If[Head[exp]\  =!= \ \ Dot, \ Return[exp]]; \n\t\t
        If[FreeAllQ[exp, \ L], \ Return[exp]]; \n\t\t
        exp\  = \ 
          exp\  /. \ 
            \(Verbatim[Dot]\)[a_, \ b_]\  :> \ 
              Sum[a[i]\ b[i], \ {i, \ 1, \ dim}]; \n\tExpandAll[\ exp]]\ 
    \)\)], "Input",
  CellLabel->"In[39]:="],

Cell[BoxData[
    \(\(\(ZeroDots = 
      \ {\(Verbatim[Dot]\)[0, \ _]\  -> \ 0, \ 
        \(Verbatim[Dot]\)[_, \ 0]\  -> \ 0}; \)\t\t\)\)], "Input",
  CellLabel->"In[40]:="],

Cell[BoxData[
    \(\( (*\ 
      Counts\ recursively\ the\ number\ of\ occurrences\ of\ var\ in\ expr\ 
        with\ allowance\ for\ multiplicities\ via\ \(Power . \)*) \n\t
    CountWithMultiplicity[expr_, \ var_]\  := \ 
      Module[{count, e = expr, v = var}, \n
        count[u_, \ v]\  := \ 0\  /; \ FreeQ[u, \ v]; \n
        count[v, \ v]\  := \ 1; \ncount[v]\  := \ 0; \n
        count[Power[x_, \ u_], \ v]\  := \ u\ \ count[x, \ v]; \n
        count[h_[a_, \ b___], \ v]\  := 
          count[a, \ v] + count[b, \ v] + \ count[h, \ v]; \n
        count[a_, \ b___, \ v]\  := \ count[a, \ v] + count[b, \ v]; \n
        count[e, \ v]\n\t\t]\)\)], "Input",
  CellLabel->"In[41]:="],

Cell[BoxData[
    \( (*\ 
      GetWeight[term_, \ expr_]\  := \ 
        Module[{e\  = \ term, \ L, \ weight = 0\ \ }, \n\t\t
          L\  = \ Cases[e, \ expr^_, \ {0, \ Infinity}]; \n\t\t\t\t
          L\  = \ L\  //. \ v_^u_\  :> \ u; \n\t\t\tweight\  = \ Plus\ @@\ L; 
          \n\t\te\  = \ e\  /. \ expr^_ -> \ 1; \n\t\t
          L\  = \ Cases[e, \ expr, \ {0, \ Infinity}]; \n\t\t
          weight\  += Length[L]\n\t]\ *) \)], "Input",
  CellLabel->"In[42]:="],

Cell[BoxData[
    \(\(\t\ 
    ChooseEvenTerm[term_, \ var_]\  := \ 
      Module[{t = term, v = var}, \n\t\t
        If[\ OddQ[CountWithMultiplicity[t, \ v]], \ Return[0]]; \n\t\tt]\t\t
    \)\)], "Input",
  CellLabel->"In[43]:="],

Cell[BoxData[
    \(ChooseEven[e_, \ var_]\  := \ 
      Module[{}, \ \(\((ChooseEvenTerm[#, \ var])\)&\)\ /@\ e]\)], "Input",
  CellLabel->"In[44]:="],

Cell[BoxData[
    \(\(\tIntDotsTerm[Exp[g_]\ f_, \ var_, \ dim_]\  := \ 
      Module[{ans, count, \ f1\  = \ f, \ g1, \ a, \ b, \ c, \ d = dim, 
          index, len1, \ len2, \ lendouble, \ Ldouble, L1, L2, term, w, \ \ 
          x = var, \ y, \ z}, \n\t\t\t\tg1\  = \ Expand[g]; \n\t\t
        a\  = \ Coefficient[g1, \ x^2]; \n\t\tb\  = \ Coefficient[g1, \ x]; \n
        \t\tc\  = \ g1 //. \ x -> \ 0; \n\t\t
        ans\  = Pi^\((d/2)\)\ 
            Exp[c + b^2/\((4 \((\(-\ a\))\))\)]/\((\(-a\))\)^\((d/2)\); \n\t\t
        If[FreeQ[f1, \ x], \ \ Return[ans\ \ f1]]; \n\t\t
        count\  = \ CountWithMultiplicity[f1, \ x]; \n\t\t
        If[OddQ[count], \ Return[0]]; \n\t\tterm\  = \ FreePartTerm[f1, \ x]; 
        \n\t\tIf[Head[term]\  === \ FreePartTerm, \ term\  = \ 1]; \n\t\t
        f1\  = \ f1/term; \n\t\tf1\  = List[f1]; \n\t\t\t
        f1\  = \ f1\  //. \ 
            Power[u_, \ v_Integer]\  :> \ ListIterate[u, \ v]; \n\t\t
        f1\  = \ Flatten[f1]; \n\t\tL1\  = \ Position[f1, \ x[_]]; 
        \  (*\ x[1], \ e . g . \ *) \n\ \ \ \ L1\  = \ Extract[f1, \ L1]; \n
        \ \ \ \ L2\  = \ 
          Position[f1, \ 
            \(Verbatim[Dot]\)[x, \ u_]\  | \ \(Verbatim[Dot]\)[u_, \ x]]; \n
        \ \ \ \ L2\  = \ \ \ Extract[f1, \ L2]; 
        \  (*\ x . k\ or\ k . x\ or\ x . x\ *) \n\t\t
        L2\  = DeleteCases[L2, \ \ \(Verbatim[Dot]\)[x, \ x]]; \n\t\t
        L2\  = \ L2\  /. 
            \ {\(Verbatim[Dot]\)[x, \ u_] -> \ u, \ 
              \(Verbatim[Dot]\)[u_, \ x] -> \ u}; \n\t\t
        Ldouble\  = \ Position[f1, \ x . x]; \n\t\t
        Ldouble\  = \ Extract[f1, \ Ldouble]; \  (*\ x . x\ only\ *) \n\t\t
        len1 = Length[L1]; \ len2\  = \ Length[L2]; \t\t
        lendouble\  = \ Length[Ldouble]; \n\t\t
        If[count\  == \ 2, \ \ 
          If[\ lendouble\  == \ 1, \ 
            Return[d\ term\ /\((2\ \((\(-a\))\))\)\ ans]];  (*\ x . x\ *) \n
          \t\t\t\n\t\t\t\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \ 
          If[len1\  == \ 1, \ index\  = \ \ L1[\([1]\)]\  /. x[u_] :> \ u\ ; 
            \n\t\t\t\t\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
            y\  = \ 
              L2[\([1]\)]\  /. 
                \ {\(Verbatim[Dot]\)[\ x, \ v_]\  :> \ v, \ 
                  \(Verbatim[Dot]\)[u_, x]\  :> \ u}; \n
            \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
            Return[term\ \ y[index]\ ans/\((2\ \((\(-a\))\))\)]]; 
          \  (*\ x[1]\ x . k\ *) \n\t\t\t\n
          \t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \ 
          If[len1\  == \ 2, \ index\  = \ L1[\([1]\)]\  /. \ x[u_]\  :> \ u; 
            \n\t\t\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ y\  = \ L1[\([2]\)]\  /. \ x[u_]\  :> \ u; \ \n
            \t\t\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
            z\  = \ \[Delta][Sequence\ @@Sort[{index, \ y}]]\ ; \n
            \t\t\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ Return[term\ \ z\ ans/\((2\ \((\(-a\))\))\)]]; \  (*\ x[1]\ x[2]\ *) \n
          \t\t\t\n\t\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \ \ \ \ \ 
          If[len2 == 2, \ y\  = \ L2[\([1]\)]; \ z = L2[\([2]\)]; \n
            \t\t\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
            w\  = \ Dot[Sequence\ @@\ Sort[{y, \ z}]]; \ \ \ \n
            \t\t\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \ \ \ \ \ Return[term\ w\ ans/\((2\ \((\(-a\))\))\)]]; 
          \  (*\ x . k\ x . a\ *) 
            \ \ \ \ \ \ \ \t\t\ \ \t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \n\t\t\t\t]; \n\t\t\t\t (*\ Now\ count\  = \ 4\ *) \n\t\t\t\t
        Return[\ \((\ 3\ d\  + \ \ d\ \((d - 1)\))\)\ \ term\ /\((4\ a^2)\)\ 
            ans]\ ]\ \n\t\t\)\)], "Input",
  CellLabel->"In[45]:="],

Cell[BoxData[
    \(FreePartTerm[e_Times, \ x_]\  := \ 
      Module[{L}, \n\t\tL\  = \ List\ @@\ e; \n\t\t
        L\  = \(\((If[\(! FreeQ[#, \ x]\), 1, \ #])\)&\)\ /@\ L; \n\t\t
        Times\ @@\ L]\)], "Input",
  CellLabel->"In[46]:="],

Cell[BoxData[
    \(ListIterate[e_, \ n_Integer\  /; \ Positive[n]]\  := \ 
      Module[{L\  = \ {}}, \n\t\tDo[L = \ {L, \ e}, \ {n}]; \n\t\tFlatten[L]]
        \)], "Input",
  CellLabel->"In[47]:="],

Cell[BoxData[
    \(DoTheqprimeIntegration[ee_]\  := \ 
      Module[{ans, \ exp\  = \ ee, \ \ factor, func, \ work, \ L1\ , \ L2, \ 
          L3, \ XXT\ \ }, \n\t
        \(factor\  = \ 
          Exp[\(-x\)\ \[Alpha][23]\ \[Alpha][14]\ q^2/\[CapitalSigma]]; \); 
        \n (*\tfactor\  = \ \ 
            \(-e^4\)\ \ \[Alpha][23]\ \[Alpha][14]\ /\ \[CapitalSigma]\ 
              Exp[\(-x\)\ \[Alpha][23]\ \[Alpha][14]\ q^2/\[CapitalSigma]]; \n
          \t\tfactor\  = \ 
            factor/\((\ \((4\ Pi\ \[CapitalSigma])\)^\((d/2)\)\ \((2\ Pi)\)^d)
                \); \n\t\tfactor\  = \ factor/xprime^2; \ *) \t\t\n\n\ 
        \[CapitalGamma][\[Rho], \ \[Sigma]]\  = \ 
          factor\ \ G[\[Rho], \ \[Sigma]]; \ \n
        XXT\  = \ 
          \[CapitalGamma][\[Rho], \ \[Sigma]]\ \  /. \ u_\ E^v_\  :> \ v; \n
        XXT = XXT\  //. ea; \nXXT\  = \ Simplify[ExpandAll[XXT]]; \n
        XXT\  = \ XXT\  //. \ ca; \t\t\n
        \[CapitalGamma][\[Rho], \ \[Sigma]]\ \  = \ 
          \[CapitalGamma][\[Rho], \ \[Sigma]]\ \  /. \ 
            u_\ E^v_ :> \ u\ E^XXT; \n
        XXT\  = \ 
          \[CapitalGamma][\[Rho], \ \[Sigma]]\ \  /. \ u_\ E^v_\  :> \ v; \n
        XXT\  = \ ctsf[XXT, \ q]; \n
        XXT\  = \ XXT\  /. \ \((q + _)\)^2 -> \ qprime^2; \n\t\t\t
        ans\  = \ 
          exp\  //. 
            \ {q -> \ qprime\  - \ \[Alpha]\ k, \ 
              Q -> \ \[Alpha]hat[23]\ qprime + \ z\ k, \ 
              q[u_]\  :> \ qprime[u]\  - \ \[Alpha]\ k[u], \ 
              Q[u_]\  :> \ \[Alpha]hat[23]\ qprime[u]\  + \ z\ k[u]}; \n
        ans\  = \ ExpandDots[ans, \ {qprime, \ k}]; \nwork\  = \ ans; \n\t\t
        If[Head[work]\  === \ Plus, \ L1\  = \ List\ @@\ work, \ 
          L1\  = \ {work}]; \nL2\  = ChooseEven[L1, \ qprime]; \n
        L3\  = \ DeleteCases[L2, \ 0]; \n
        XXT\  = \ XXT\  /. \ x -> \ \(-1\) + 1/xprime; \n
        XXT\  = ExpandAll[XXT]; \n\t\t\t
        \[CapitalGamma][\[Rho], \ \[Sigma]]\ \  = \ 
          \[CapitalGamma][\[Rho], \ \[Sigma]]\ \  /. \ 
            u_\ E^v_ :> \ u\ E^XXT; \n\t\t
        XXT\  = \ XXT\  /. \ Coefficient[XXT, \ qprime^2] :> \(-\ y\); \n\t\t
        func\  = \ \ \ E^XXT\ L3; \n
        ans\  = \ \(\((IntDotsTerm[#, \ qprime, \ d])\)&\)\ \ /@func; \n\t\t
        ans\  = \ Plus\ @@\ ans; \n\t\t
        ans\  = \ ans\  /. \ E^u_ :> \ E^Simplify[ExpandAll[u]]; \n\t\t
        ans\  = \ \ 
          Collect[ans, 
            \ {E^_, \ _^\((\(-d\)/2)\), \ \[Delta][a_, \ b_], \ \ 
              \[Alpha][14]\ \[Alpha][23], Pi^_, \(Verbatim[Dot]\)[k, \ k]^_}]]
        \)], "Input",
  CellLabel->"In[48]:="],

Cell[BoxData[
    \(\(JacDet[y_List, \ x_List]\  := \ 
      Module[{det, \ dim, element, i, \ j, \ M, \ X, \ Y}, \n\t\t
        If[Length[x]\  != \ Length[y], \ Print["\<Not a square matrix\n\>"]; 
          Return[]]; \n\t\tdim\  = \ Length[x]; \n\t\ 
        M\  = \ Table[\ D[y[\([i]\)], \ x[\([j]\)]], \ {i, \ 1, \ dim}, 
            \ {j, \ 1, \ dim}]; \n\tdet\  = \ Det[M]\n
        \t\t (*\ det\  = \ FullSimplify[det]\ *) ]\t\ \n\t\t\)\)], "Input",
  CellLabel->"In[49]:="],

Cell[BoxData[
    \(\( (*\ 
      Works\ with\ symbolic\ powers\ var^pow . \ \ FullSimplifies\ the\ 
        coefficient\ of\ var^pow . \ \ \ Faster\ than\ FullSimplify\ *) \n
    SimplifyTheCoefficients[exp_, \ var_]\  := \ 
      Module[{\ ans\ , expr, \ c, \ i, \ len, L, \ Lexpr, \ power, Q, u, \ v
          \ }, \n\t\ \ ans\  = \ 0; \n\t\tv\  = \ var; \n\t\t\ 
        expr\  = \ Expand[exp, \ v]; \n\t\texpr\  = \ Collect[expr, \ v^_]; \n
        \t\tIf[Head[expr]\  === \ Plus, \ Lexpr\  = \ List\ @@\ expr, \ 
          Lexpr\  = \ {expr}]; \ \n
        \t\t (*\tL\  = \ Cases[expr, \ v^u_, \ {0, \ Infinity}]; \ 
          NO . \ \ WANT\ OUTERMOST\ LEVEL\ ONLY\ *) \n\t\t\t
        L\  = \ Cases[expr, \ v^u_, \ {0, 2}]; \n\t\t\t
        L\  = \ L\  /. \ v^u_ :> \ u; \n\t\t\tlen\  = \ Length[L]; \n\t\t
        For[i\  = \ 1, \ i\  <= \ len, \ \(i++\), \n\t\t\t\ \ \ \ \ \ \ \ 
          \(\(\(u\  = \ L[\([i]\)]; \); \); \); \n
          \t\t\t\ \ \ \ \ \ \ \  (*\ 
            c\  = \ Coefficient[Lexpr[\([i]\)], \ v^u]; \ *) \n\t\t\t\t\t
          c\  = \ GetMeList[expr, \ v^u]; \n\t\t\t
          If[Length[c] == 0, \ c = 0, \ c = c[\([1]\)]]; \n\t\t\t\t\t\t\ \ \ 
          c\  = \ FullSimplify[c]; \n\t\t\t\t\ \ \ \ \ \ \ \ \ 
          ans\  = \ ans\  + \ c\ v^u; \n\t\t]; \n\t\t
        Lexpr\  = \ Lexpr\  //. \ v^u_\  :> \ 0; \ \n\t\t
        expr\  = \ Plus\ @@\ Lexpr; \n\t\t
        c\  = \ Coefficient[expr, \ v, \ 0]; \n\t\t\tc\  = \ FullSimplify[c]; 
        \n\t\t\ ans\  = \ ans\  + \ c; \n\t\t
        c\  = \ Coefficient[expr, \ v, \ 1]; \n\t\tc\  = \ FullSimplify[c]; \n
        \t\tans\  = \ ans\  + \ c\ v\ \n\t\t]\)\)], "Input",
  CellLabel->"In[50]:="],

Cell[BoxData[
    \(\(stc\  := \ SimplifyTheCoefficients; \)\)], "Input",
  CellLabel->"In[51]:="],

Cell[BoxData[
    RowBox[{
    \( (*\ Finds\ all\ occurrences\ of\ FindMe\ 
          \((some\ symbol\ or\ expression)\)\ in\ exp\ and\ replaces\ FindMe\ 
          by\ a\ marker\ MarkMe . \n\tThis\ is\ useful\ after\ Collect\ to\ 
          extract\ the\ coefficient\ of\ FindMe\ 
          \((then\ we\ would\ ordinarily\ set\ MarkMe\  = \ 1)\) . \ \ Then\ 
          we\ would\ use\ \(GetMeList[ ... ]\)[[1]] . \ \ Remember, \ 
        GetMeList\ returns\ a\ list\ *) \), "\n", 
      \( (*\ Further\ 
          \(note : \ 
            Applying\ Coefficient[]\ will\ destroy\ any\ previously\ performed
              \ simplification . \ Since\ simplication\ is\ expensive\ 
              \((especially\ FullSimplify)\)\ we\ may\ want\ to\ preserve\ 
              it . \ \ Hence\ the\ need\ for\ GetMeList\ and\ GetMeCollect
              \)\ *) \), "\n", "\n", 
      RowBox[{"(*", " ", 
        RowBox[{
          RowBox[{
          "Further", " ", "note", " ", "on", " ", "why", " ", "Cases", " ", 
            "is", " ", 
            RowBox[{"inadequate", ":", "\n", "\t\t\t", 
              RowBox[{
              \(Cases[3\ asd\ XX\ x1\ Log[XX], \ XX\ Log[XX], 
                  \ {0, \ Infinity}]\), "\n", "gives", " ", \({}\), "\n", 
                \(FullForm[3\ asd\ XX\ x1\ Log[XX]]\), " ", "is", "\n", 
                TagBox[
                  StyleBox[\(Times[3, asd, x1, XX, Log[XX]]\),
                    ShowSpecialCharacters->False,
                    ShowStringCharacters->True],
                  FullForm]}]}]}], "\n", 
          \(GetMeList[3\ asd\ XX\ x1\ Log[XX], \ u_\ Log[u_]]\nreturns
            \ {3\ asd\ x1}\), "\n", 
          \(Cases[3\ asd\ XX\ x1\ Log[XX], \ u_\ Log[u_], \ {0, \ Infinity}]\ 
            returns\n{}\)}], "\n", "\t\t", "*)"}], "\n", "\n", 
      \( (*\ USE\ ONLY\ AFTER\ \(COLLECT\ !!\)\n\tTHIS\ FUNCTION\ ACTS\ ON\ 
            THE\ OUTERMOST\ LEVEL\ ONLY . \n\tExample\ of\ proper\ 
            \(usage : \n (*\te\  = \ Collect[expr, \ FindMe^_]; \ *) \n
              \t\t\t\te\)\  = \ Collect[expr, \ FindMe]; \n\t
        GetMeList[expr, \ FindMe^pow]; \n\t
        This\ will\ find\ the\ coefficient\ of\ FindMe^pow\ without\ 
          disturbing\ any\ previous\ simplification\n\t\ *) \), "\n", 
      \(GetMeList[exp_, \ FindMe_, \ MarkMe_:  1]\  := \ 
        Module[{e = exp, \ find = FindMe, \ L}, \n\t\t
          If[Head[e] === Plus, \ L\  = \ List@@e, \ L\  = \ {e}]; \n\t\t\t\t
          L\  = \(\((If[FreeQ[#, \ find], Null, \ \ #\  /. \ find -> MarkMe])
                  \)&\)\ /@\ L; \n\t\t\ \ \ \ 
          L\  = \ DeleteCases[L, \ Null]\n (*\t
            If[Length[L] == 0, \ 0, \ L[[1]]]\ *) \n\t]\)}]], "Input",
  CellLabel->"In[52]:="],

Cell[BoxData[
    \(GetMeCollect[exp_, \ FindMe_, \ MarkMe_:  1]\  := \ 
      Module[{e, find = FindMe, \ L, \ XXT}, \n\t\te = e /. \ find -> \ XXT; 
        \  (*\ To\ avoid\ problems\ with\ Collect . \ If\ find\  = \ 1 - u, \ 
          for\ instance, \ 
          Collect\ may\ fail\ to\ do\ the\ expected\ simplification\ of\ 
            things\ like\ \((1 - u)\)\ \((1 + \((1 - u)\)^\((\(-1\)/2)\))\), 
          \ although\ it\ will\ do\ this\ for\ an\ atom\ such\ as\ 
            XXT . \ \ See\ my\ file\ 
            "\<Why I have trouble with Collect.nb\>" . \n\t\t\tThis\ problem\ 
            with\ Collect\ can\ be\ avoided\ by\ specifying\ a\ 
            \(level . \)\ *) \n\t
        e\  = \ Collect[e, \ \ find]\  /. \ XXT -> \ find; \ \t\n\t\t
        L\  = \ GetMeList[e, \ FindMe, \ MarkMe]; \n\t\t\t
        If[Length[L] == 0, \ 0, \ L[\([1]\)]]\n\t]\)], "Input",
  CellLabel->"In[53]:="],

Cell[BoxData[
    \(JD\[Beta]\  = \ 
      JacDet[{\[Beta]\ u, \ \((1 - \[Beta])\) v, \ 
          \((1 - \[Beta])\) \((1 - v)\)}, \ {\[Beta], \ u, \ v}]; \n
    JD\[Beta]\  = \ \(-JD\[Beta]\); 
    \  (*\ JD\[Beta]\  = \ \[Beta]^2\  - \ \[Beta]\ in\ the\ previous\ step, 
      \ and\ we\ want\ the\ absolute\ value\ of\ the\ Jacobian\ in\ order\ to
        \ represent\ the\ transformation\ of\ the\ volume\ element\ in\ the\ 
        integrals . \ \ Always, \ 0 <= \ \[Beta]\  <= \ 1\ *) \n
    JD\[Alpha] = 
      JacDet[{t\ \[Alpha]hat[1], t\ \[Alpha]hat[2], t\ \[Alpha]hat[3], \ 
          t\ \((1 - \[Alpha]hat[1] - \[Alpha]hat[2] - \[Alpha]hat[3])\)}, 
        \ {\[Alpha]hat[1], \ \[Alpha]hat[2], \ \[Alpha]hat[3], \ t}]; 
    \n (*\ t\ is\ synonymous\ with\ \[CapitalSigma]\ *) \n
    To\[Beta]uvRules\  = 
      \ {\[Alpha][1] -> \ \[Beta]\ u, \ 
        \[Alpha][2] -> \ \((1 - \[Beta])\)\ v, \ 
        \[Alpha][3] -> \ \((1 - \[Beta])\) \((1 - v)\), \ 
        \[Alpha][4] -> \ \[Beta] \((1 - u)\), \[Alpha][14] -> \ \[Beta], \ 
        \[Alpha][23] -> \ 1 - \[Beta], \ 
        \[Alpha][12] -> \ \[Beta]\ u + \((1 - \[Beta])\)\ v, \ 
        \[Alpha][34] -> \ 
          \((1 - \[Beta])\) \((1 - v)\)\  + \ \[Beta] \((1 - u)\)}; \n
    TotRules\  = 
      \ {\[CapitalSigma] -> \ t, \ \[Alpha][j_] :> \ t\ \[Alpha][j]}; \n\t
    tOutRules\  = 
      \ {t\ a_\  + \ t\ b_\  :> \ t \((a + b)\), \ 
        t\ a_\  - \ t\ b_\  :> \ t\ \((a - b)\)}; \)], "Input",
  CellLabel->"In[54]:="],

Cell[BoxData[
    \(protectrule\  = \ {\[Alpha][u_] -> \ Protect\[Alpha][u]}; \n
    finalrules\  = 
      \ {\[Alpha] -> \ 
          xprime\ \((\[Alpha][1] \[Alpha][3] - \[Alpha][2] \[Alpha][4])\)/
              \((\[Alpha][14]\ \[Alpha][23])\), \n
        \t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
        z -> \ \[Alpha][34]/\[CapitalSigma]\  - \ 
            \((\[Alpha][1]\ \[Alpha][3]\  - \ \[Alpha][2]\ \[Alpha][4])\)\ 
              xprime/\((\[CapitalSigma]\ \[Alpha][14])\), \n\t\t\ \ \ 
        \[Alpha]hat[23] -> \ \[Alpha][23]/\[CapitalSigma]\ \ \ \ \ , 
        \ \ \ \ \ 
        y -> \ \[Alpha][23]\ \[Alpha][14]/\((xprime\ \[CapitalSigma])\)\ \n
        \t\t (*\ y\  -> \ \[Beta]\ \((1 - \[Beta])\)\ t/xprime\ *) }; \n\t\t
    unprotectrule\  = \ Protect\[Alpha] -> \[Alpha]; \)], "Input",
  CellLabel->"In[55]:="],

Cell[BoxData[
    \(SubstituteBack[exp_]\  := \ 
      Module[{ans\  = \ exp}, \n
        \t\t (*\ Use\ the\ various\ rules\ in\ code . nb\ to\ back - 
            substitute\ for\ \[Alpha], \ z, \ \[Alpha]hat[23], \ 
          and\ y\ in\ terms\ of\ the\ original\ \[Alpha][j]' s\ *) \n\t\t\ 
        ans\  = \ ans\  /. \ protectrule; \nans\  = \ ans\  /. \ finalrules; 
        \nans\  = \ ans\  /. \ unprotectrule; \n\t
        \(\(ans\t\  = \ ans\ \  /. \ \(Verbatim[Dot]\)[k, \ k] :> \ k^2; \); 
        \); \n (*\t
          ans\  = \ 
            ans\  /. \ 
              \[Alpha][n_Integer]\  :> \ 
                \[Alpha][Floor[n/10]] + \[Alpha][Mod[n, \ 10]]\  /; \ 
                  n\  > \ 9; \ *) \n\t\tans\  = \ ans\  /. TotRules; \n
        \n (*\t\tans\  = \ ans\  /. \ tOutRules\ *) \n (*\t
          ans\  = \ FullSimplify[ans]\ *) \nans\t]\)], "Input",
  CellLabel->"In[56]:="],

Cell[BoxData[
    \(FinishOff[exp_]\  := \ 
      Module[{\ ans, c0, \ cminus1, \ cminus2, coefs, \ 
          CoefficientOftInExponential, expr1, \ expr2, \ factor, \ Lsimp, \ \ 
          q1, q2, \ q3, \ q4, q5, \ result, \ rule, simp1, \ simp2, \ 
          Expofactor, \ YYT\ \ }, \n\t\tq1\  = \ exp; \n\t\t
        Expofactor\  = 
          \(Cases[\ q1\ , \ E^u_, \ {0, \ Infinity}]\)[\([1]\)]\  /. \ 
            E^v_ -> \ v; \n\t\t
        Expofactor\  = \ 
          Simplify[SubstituteBack[Expofactor]\  /. \ To\[Beta]uvRules]; \n\t\t
        Expofactor\  = \ Simplify[Expofactor]; \n\t\t
        \(q1\  = \ q1\  /. \ E^_ -> \ 1; \); 
        \n (*\t\tq1\  = \ Expand[q1]; \ *) \n\t\tq1debug\  = \ q1; \t\t\n\t\ 
        \n\t\t\tfactor\  = \ \ 
          \(-e^4\)/\ PowerExpand[\((4\ Pi)\)\ ^\((d/2)\)\ \((2\ Pi)\)^d]; \t\n
        \t\tcoefs\  = \ GetCoefficients[q1]; 
        \ \  (*\ Get\ coef0, \ coefminus1, \ coefminus2\ *) \n\t\t
        coefs\  = \ 
          \(\((SubstituteBack[#]\  //. \ To\[Beta]uvRules)\)\ &\)\ /@\ coefs; 
        \n\t (*\ Now\ for\ some\ ugly\ but\ timely\ simplification\ *) \n
        \n (*\t\texpr1 = 
            \(1\/\(t\^2\ \[Beta]\)\) 
              \((t\^2\ u\ \((1 - v)\)\ \((1 - \[Beta])\)\ \[Beta] - 
                  t\^2\ \((1 - u)\)\ v\ \((1 - \[Beta])\)\ \[Beta])\); \n
          \ \ \ \ simp1\  = \ Simplify[expr1]; \n\ 
          coefs\  = coefs\  //. \ expr1 -> \ simp1; \n\n\ \ \ \ \ 
          expr2\  = \ \ \ \ \ 
            \((t\^2\ u\ \((1 - v)\)\ \((1 - \[Beta])\)\ \[Beta] - 
                    t\^2\ \((1 - u)\)\ v\ \((1 - \[Beta])\)\ \[Beta])\)^2/
              \((t\^4\ \((1 - \[Beta])\)\^2\ \[Beta]\^2)\); \n
          simp2\  = \ Simplify[expr2]; \n
          coefs\  = \ coefs\  //. \ expr2 -> \ simp2; \ *) \n\t\n\t\t\t\t
        coefs\  = \ \(\((Together[#])\)&\)\ /@\ coefs; 
        \  (*\ cancel\ common\ factors\ of\ \[Beta]\ and\ 1 - 
            \[Beta]\ from\ numerator\ and\ denominator\ *) \n
        coefsdebug\  = \ coefs; \n\t
        coefs\  = coefs\  //. \ d -> \ 4 - \[Epsilon]; \n\t\t
        coefs\  = \ 
          coefs\  //. \ 
            \(-\[Beta]\) + \[Beta]^2 -> \ 
              \(-\[Beta]\)\ Hold[\((1 - \[Beta])\)]; 
        \  (*\ This\ is\ \(essential!\)\ \n\t\tMathematica\ keeps\ trying\ to
              \ replace\ 1\  - \ \[Beta]\ by\  - \((\[Beta]\  - \ 1)\), \ 
          which\ messes\ everything\ up\ because\ I\ need\ to\ take\ 
              noninteger\ powers\ of\ 1\  - \ \[Beta], \ 
          where\ I\ know\ that\ 0\  <= \ \[Beta]\  <= \ 1. \ *) \n\t\t
        coefs\  = \ ReleaseHold[coefs]; \ \n\tcoefsdebug\  = \ coefs; \t\n\t\t
        ans\  = \ FinishOffByPowerOft[coefs, \ Expofactor]; \t\n\t\t
        ans\  = Normal[\ Series[ans, \ {\[Epsilon], \ 0, \ 2}]\ ]\  /. \ 
            e -> \ Sqrt[4\ Pi\ \[Alpha]]; \n\t\t
        ans\  = \ ans\ \  //. \ Log[k] :> \ Log[k^2]/2; \t
        \n (*\tans\  = \ FullSimplify[ans]\ \ *) \n\t\t
        ans = ans\ factor\ \  /. \ {d -> 4, \ e -> \ Sqrt[4\ Pi\ \[Alpha]]}; 
        \n\t\tans\  = \ 
          Coefficient[ans, \ \[Epsilon], \ \(-2\)]/\[Epsilon]^2\  + \ 
            Coefficient[ans, \ \[Epsilon], \ \(-1\)]/\[Epsilon]\  + \ 
            OnlyHigherThanSquares[Coefficient[ans, \ \[Epsilon], \ 0], \ k]\n
        \t\t\t]\)], "Input",
  CellLabel->"In[57]:="],

Cell[BoxData[
    \(\( (*\ 
      Limitation : \ 
        Currently\ assumes\ we\ want\ to\ get\ coefficients\ of\ powers\ of\ 
          y . \ \ In\ future, \ 
      may\ want\ to\ pass\ var\ as\ an\ argument\ to\ allow\ the\ \ \ \ same\ 
        thing\ with\ a\ general\ var . \ \ If\ var\ is\ an\ expression\ rather
        \ than\ a\ symbol, \ 
      we\ will\ need\ to\ use\ the\ XXT\ trick . \ \ See\ my\ file\ 
        "\<Why I have trouble with Collect.nb\>"\ *) \n
    GetCoefficients[expr_]\  := \ 
      Module[{ans, \ L, \ L1}, \n\t\ L\  = \ Collect[expr, \ y^_]; \ \n
        \t (*\t\t\ L\  = \ Collect[expr, \ y]; \ *) \n\t
        L1\  = \ \ Cases[L, \ y^\((\(-d\)/2)\)\ \ v_, \ {0, \ Infinity}]\ ; \n
        \t\t\tIf[L1\  == \ {}, \ L1\  = \ {0}]; \n\t\t
        ans\  = L1\  /. \ y -> \ 1; \n\t\t
        L1\  = \ \ 
          Cases[L, \ y^\((\(-1\) - d/2)\)\ \ v_, \ {0, \ Infinity}]\ \ ; \n
        \t\tIf[L1\  == \ {}, \ L1\  = \ {0}]; \n\t\t\t
        ans\ \  = \ \ Append[ans, L1\  /. \ y -> \ 1]; \n\t\t
        L1\  = \ \ 
          Cases[L, \ y^\((\(-2\) - d/2)\)\ \ v_, \ {0, \ Infinity}]\ \ ; \n
        \t\t\tIf[L1\  == \ {}, \ L1\  = \ {0}]; \n\t\t
        ans\  = \ \ \ \ Append[ans, L1\ \  /. \ y -> \ 1]; \nFlatten[ans]\n
        \t\t]\n\t\t\)\)], "Input",
  CellLabel->"In[58]:="],

Cell[BoxData[
    \(FinishOffByPowerOft[coefs_List, ArgumentOfExponential_]\  := \ 
      Module[{\ ans = 0, \ ansminus1, \ ansminus2, c, \ cL, \ coef0, \ 
          coefminus1, \ coefminus2, \ CoefficientOftInExponential, 
          ConvergentPart, d, DivergentPart, factor, \ factorminus1, \ 
          factorminus2, i, L, \ PowerOft, PowersOft, rlist, \ \ x\ \ }, \t\t\t
        \nc\  = \ ArgumentOfExponential\ /\((t\ k^2)\); \n\t\ \ 
        L\  = \ \((Expand[#, \ t]\ &)\)\ \ /@\ coefs; 
        \  (*\ Cancel\ powers\ of\ t\ in\ numerators\ and\ denominators\ *) \n
        \ \ \ For[i = 0, \ i\  > \ \(-3\), \ \(i--\), \ \n\t\t
          cL\  = \ \ GetCoefficientsList[L[\([\(-i\) + 1]\)], \ t]; \n\ \ \ \t
          While[\ Length[cL] > 0, \n\t\t\ \ \ \ \ 
            ans\  = \ 
              ans\  + \ \ 
                DoPart[i, c, \ cL[\([1, 2]\)]\ , \(-\ cL[\([1, 1]\)]\)]; \n
            \t\t\ \ \ \tcL = Rest[cL]; \n\t\t\t]; \t\n\t]; \t\t\t\t\nans\n\t]
        \)], "Input",
  CellLabel->"In[59]:="],

Cell[BoxData[
    \(\(\t (*\ 
      This\ function\ returns\ a\ list\ of\ the\ form
        \ {\ {power, \ coef\ of\ var^power\ in\ expr}, \  ... }\ for\ every\ 
        power\ of\ var\ occurring\ in\ expr\ 
        \((including\ the\ exceptional\ powers\ 0\ and\ 1)\)\ *) \n\n\t
    GetCoefficientsList[expr_, \ var_]\  := \ 
      Module[{e = expr, \ L, \ L1, power, v = var}, \n\t\t\t\t
        L1\  = Union[\ Cases[e, \ v^_, \ {0, \ Infinity}]]; \n\t\n\t\t
        L\  = \ {{0, Coefficient[e, \ v, \ 0]}, {1, \ \ 
              Coefficient[e, \ v, \ 1]}}; \n\t\t
        While[\ Length[L1]\  > \ 0, 
          power\  = \ L1[\([1]\)]\  /. \ a_^b_\  :> \ b; \t\t\t\t\n
          \t\t\t\t\t\t\ 
          L\  = \ Append[L, \ {power, \ Coefficient[e, \ v, \ power]}]; \n
          \t\t\tL1\  = \ Rest[L1]; \n\t\t\t]; \n\t\t
        L\  = \ L\  /. \ {_, \ 0}\ \  -> \ 0; \n\t\tDeleteCases[L, \ 0]\n\t]\t
    \)\)], "Input",
  CellLabel->"In[60]:="],

Cell[BoxData[
    \(\(\t\t\t\t
    DoPart[case_, expoarg_, coef_, \ r_]\  := \ 
      Module[{ans = 0, c = expoarg, core, factor, PowerOft, \ term = 0}, \t\t
        \n\t\tIf[coef\  == 0, \ Return[0]]; \n\t\t
        PowerOft\  = \ 4 - d - r\  + case\  /. \ d -> \ 4 - \[Epsilon]; \t\t\n
        \t\tcore\  = \ \t
          \((\[Beta] \((1 - \[Beta])\))\)^\((2 - d/2)\)\  /. \ 
            d -> \ 4 - \[Epsilon]; \n\t\t
        factor\  = \ 
          coef\ \ xprime^\((d/2\  - 2 - case)\)\  /. \ d -> \ 4 - \[Epsilon]; 
        \n\t\t\tIf[case\  == \ 0, \ core\  = \ core\  /. \ \[Epsilon] -> \ 0; 
          \n\t\t\t\tIf[r == 2, \ term = \ \(-\[Epsilon]\)\ Log[k^2]]; \n
          \t\t]; \n\t\t\t\t\tfactor\  *= \ \((1 + term)\); \n\t\t\t\n
        \t\t\t\t\t\tfactor\  *= \ \ \ \ \ c^\((\(-PowerOft\) - 1)\); \n
        \t\t\t\tfactor\  = \ factor\  /. \ \[Epsilon] -> \ 0; \n\t\t
        ans\  = \ core\ factor\ \((\[Beta] \((1 - \[Beta])\)\ )\)^case; \n\t\t
        ans\  = \ PowerExpand[ans]\  /. \ u_^v_ :> \ u^Expand[v]; \n\t\t
        ans\  = \ ans\  /. \ u_^v_\ w_^v_\  :> \ \((u\ w)\)^v; \n\t\t
        ans\  = \ ans\  /. \ u^v_ :> \ u^Expand[v]; \t\t\t\t\t\n\t\t\t\t
        ans\  = \ 
          ans\  /. \ 
            \((\[Beta] \((1 - \[Beta])\))\)^\((\(-1\) + \[Epsilon]/2)\)\  -> 
              \ \[Beta]^\((\(-1\) + \[Epsilon]/2)\)\  + \ 
                \((1 - \[Beta])\)^\((\(-1\) + \[Epsilon]/2)\); \t\t\t\n
        \ \ \ \ \ \ \ \ \ \ ans\  = \ Expand[ans, \ \[Beta]]; \t\t\n\t\t
        ans\  = \ 
          MyIntegrate[ans, \ \ {xprime, \ 0, \ 1}, \ \ {u, \ 0, \ 1}, 
            \ {v, \ 0, \ 1}, \ {\[Beta], \ 0, \ 1}]; \n\t\t\t\t
        ans\  *= \ \((k^2)\)^\((\(-PowerOft\) - 1)\) Gamma[PowerOft + 1]\ ; \ 
        \n\t\t\t\t
        ans = \ Normal[Series[\ ans\ , \ {\[Epsilon], \ 0, \ 0}]]\ ; \n\t\tans
        \n\t\t]\)\)], "Input",
  CellLabel->"In[61]:="],

Cell[BoxData[
    \(MyIntegrate[expr_, \ vars__]\  := \ 
      Module[{eL\  = \ Expand[expr], vL = List[vars]}, \n\t\t
        If[Head[eL] === Plus, \ eL\  = \ List\ @@\ eL, \ eL = {eL}]; \t\n\t\t
        \(\(vL\  = \ \(\((#[\([1]\)])\)&\)\ /@\ vL; \); \); \n
        \(eL\  = 
          eL\  /. \ 
            u_^v_\  :> \ 
              TooHard[u^v]\  /; \ 
                Bad[u, \ vL] && 
                  \((\((\ Head[v]\  =!= \ Integer)\)\  || \ Negative[v])\); 
        \); \t\n\t\t\(\(eLdebug\  = \ eL; \); \); \ \n\t\t\n
        \t\t (*\teL\  = \ 
            \(\((If[FreeQ[#, \ TooHard], DoTheIntegrals[#, \ vars], \ #])\)&
                \)\ /@\ \ eL; \ \ \ 
          RESTORE\ THIS\ LATER\ We\ want\ to\ test\ DoTheIntegrals\ now\ by\ 
            replacing\ it\ by\ Integrate\ *) \n\t\t\t
        eL\  = \ \(\((
                If[FreeQ[#, \ TooHard], 
                  Integrate[#, \ vars, \ Assumptions -> \ \[Epsilon] > 0], 
                  \ #])\)&\)\ /@\ \ eL; \n\t\t\t\t
        eL\  = \ eL //. \ TooHard[w_] :> \ IntYYT[\ w, \ vars]; \n\t\t\t
        eL\  = \ eL\  //. \ 
            a_\ IntYYT[b_, w__] :> \ 
              IntYYT[a\ b, \ w]\  /; \ \(! FreeAllQ[a, \ vL]\); \n\t\t
        eL\  = \ eL\  //. \ \ \ 
            IntYYT[a_\ \ b_, \ w__]\  :> \ 
              a\ IntYYT[b, \ w]\  /; \ FreeAllQ[a, \ vL]; \n\ \ \ \ \ \ \ 
        eL\  = \ Plus\ @@\ eL; \n\t (*\t\t\teL\  = HandleIntYYT[eL]; \ \ *) \n
        \t\teL\ \n\t\t]\)], "Input",
  CellLabel->"In[62]:="],

Cell[BoxData[
    \(\( (*
      \(\(\ Limitations : \ \ 1)\)\ expr\ must\ be\ a\ polynomial\ in\ vars, 
          \ Log[vars], \ and\ Log[1 - vars] . \ \ However, \ 
          Mathematica\ can' t\ handle\ vars^a\ Log[1 - vars]^b\ for\ general\ 
            positive\ integers\ a, \ b\n\t2)\)\ Integration\ ranges\ specified
        \ by\ vars\ must\ all\ be\ from\ 0\ to\ 1\n\t\t\ *) \n (*\ 
      This\ code\ also\ illustrates\ an\ important\ use\ of\ Hold\ to\ 
        suppress\ a\ rule\ that\ follows\ another\ conditional\ rule\ which\ 
        is\ not\ executed . \ \ \(Example : \ Log[x]^x\ matches\ Log[x]^a_\), 
      \ but\ the\ condition\ FreeQ[a, \ var = x]\ fails; \ 
      we\ certainly\ do\ not\ want\ the\ next\ rule\ Log[x] -> 
        \(-1\)\ to\ go\ into\ effect\ in\ such\ a\ case\ *) \n\n\n
    DoTheIntegrals[expr_, \ vars__]\  := \ 
      Module[{ans, \ e = Expand[expr], \ ints, \ L, len, \ \ rules, u, \ v, \ 
          a, \ b, var, \ vL, \ XXT\ }\n\t\t, \ \t\t
        \n (*\te\  = \ 
            e\  //. \ {Log[a_\ b_]\  :> \ Log[a] + Log[b], \ 
                Log[a_/b_] :> \ Log[a] - Log[b]}; \n\t\t
          Print[\ \ \ \ \ "\<In DoTheIntegrals after Log expansion, e is \>", 
            \ e]; \n\t\t\ 
          NOTE : \ this\ doesn' t\ work\ for\ 
              Log[\(-\((\(-1\) + v)\)\) v]\ *) \t\n
        vL\  = \ \(\((#[\([1]\)])\)&\)\ /@\ List[vars];  (*\ 
          The\ names\ of\ the\ variables\ *) \n\t\tlen\  = \ Length[vL]; \n
        \t\tWhile[\(len--\) > 0, \ var\  = \ vL[\([1]\)]; \n\t\t\t\t
          If[\((L = CheckDivergence[e, \ var])\)\  =!= \ 1, 
            \  (*\ NOTE : \ \( != \ is\ wrong\)\ *) \ 
            Print[\ "\<Too Divergent.  var is \>", \ var, \ 
              "\< power list is \>", \ L]; Return[TooDivergent]]; \t\t\t\n
          \t\t\t (*\ 
            This\ forces\ all\ parts\ more\ divergent\ than\ 
              var^\((\(-1\) + \[Epsilon]/2)\)\ or\ 
              \((1 - var)\)^\((\(-1\) + \[Epsilon]/2)\)\ to\ return\ 
              TooDivergent\ and\ an\ error\ message, \ 
            which\ is\ rather\ drastic\ *) \n\t\t\t\t\ 
          e = e //. \ \(-1\) + var -> \ \(-Hold[1 - var]\); 
          \n (*\t\t\te\  = \ ReleaseHold[e]; \ *) \n\t\t\tvL = Rest[vL]; \n
          \t\t\t]; \n\t
        e\  = \ 
          e\  //. \ {Log[a_\ b_]\  :> \ Log[a] + Log[b], \ 
              Log[a_/b_] :> \ Log[a] - Log[b]}; \n\t\t
        vL\  = \ \(\((#[\([1]\)])\)&\)\ /@\ List[vars]; \n\t\t
        len\  = \ Length[vL]; \n\t\t\n
        While[\(len--\) > 0, \ var := vL[\([1]\)]; \t\t\n\t\t
          L\  = \ {var^u_\ Log[1 - var], var^u_\ Log[1 - var], \n
              \t\t\t\t\t\t\ var^u_\ Log[var], \n\t\t\t\t\t\t\ 
              var^u_\ Log[var]}; \n\ \ \ \ 
          L\  = \ Append[L, \ L /. \ var -> \ 1 - var]; \n\t\t\n\ \ \ \ \ \ 
          ints\  = 
            \ {\(1\/\(1 + u\)\) \((EulerGamma + PolyGamma[0, 2 + u])\)\  /; \ 
                FreeQ[u, \ var]\ \ , \n\t\t\t\t\t
              Hold[var^u\ Log[1 - var]]\  /; \ \(! FreeQ[u, \ var]\), 
              \t\t\t\t\t\t\n\t\t\t\t\t\t\ 
              \(-1\)/\((1 + u)\)^2\ \  /; \ FreeQ[u, \ var], \n\t\t\t\t\ \ 
              Hold[\ var^u\ Log[var]]\  /; \ \(! FreeQ[u, \ var]\)}; \n
          \ \ \ \ ints\  = \ Append[ints, ints]; 
          \  (*\ \(Symmetric\ under\ var < \) -> 1 - var\ *) \t\t\n\ \ \ \ 
          rules\  = \ Thread[RuleDelayed[L, Evaluate[ints]]]; \n\t\t\t
          rules\  = \ 
            Append[rules, 
              \ {var\ Log[var]\  -> \ \(-1\)/4, \ 
                Log[var]^a_\  :> \ 
                  \((\(-1\))\)^a\ Gamma[1 + a]\  /; \ FreeQ[a, \ var], \n
                \t\t\t\t\t\t\t\t
                Log[var]^a_\  :> \ 
                  Hold[Log[var]^a]\  /; \ \(! FreeQ[a, \ var]\), 
                Log[var] -> \ \(-1\), \ Log[1 - var] -> \ \(-1\)}]; \n\t\t\t
          rules\  = \ Flatten[rules]; \n\t\t\t\te\  = \ Collect[e, L]; \n\t
          e\  = \ e\  /. \ rules\ ; \n\tvL\  = \ Rest[vL]; \n\t\t\t\t
          e\  = \ ReleaseHold[e]; 
          \ \  (*\ Necessary\ to\ do\ this\ here, \ 
            else\n\t\t\t\tthe\ Hold\ will\ prevent\ var\ from\ being\ updated
              \ *) \n\t\t\t]; \t\t\t\n\t\t
        vL\  = \ \(\((#[\([1]\)])\)&\)\ /@\ List[vars]; \n\t\t
        len\  = \ Length[vL]; \ \n
        While[\(len--\) > 0, \ var := vL[\([1]\)]; \n\t\t\t
          e\  = \ e\  /. \ 1 - var -> \ XXT; 
          \  (*\ This\ XXT\ trick\ is\ \(necessary\ --\)\ see\ the\ file\ 
              "\<Why I have trouble with Collect.nb\>"\ *) \n\t
          e\  = \ Collect[e, \ {var^_, XXT^_}]\  /. \ XXT -> \ 1 - var; \n
          \t\t\te\  = 
            e /. {\ var^a_\ \((1 - var)\)^b_\  :> 
                  \(Gamma[1 + a]\ Gamma[1 + b]\)\/Gamma[2 + a + b], \ 
                var\ \((1 - var)\)^a_ :> \ 1\/\(2 + 3\ a + a\^2\)\ , \ 
                var^a_\ \((1 - var)\) :> \ 1\/\(2 + 3\ a + a\^2\)\ }\ ; \t\t\t
          \n\t\t\t\n\t\t\t
          e\  = e /. {\ var^u_ :> \ 1/\((1 + u)\), \ 
                \((1 - var)\)^u_\  :> \ \ 1/\((1 + u)\)}\ ; \n\t\t
          e\  = \ e /. \ 1 - \ var -> \ 1/2\ \ \ ; \n\t\t\t
          e\  = \ e\  /. \ var\  -> \ 1/2; \t\n\tvL = Rest[vL]; \n\t\t\t\t]; 
        \t\ne]\)\)], "Input",
  CellLabel->"In[63]:="],

Cell[BoxData[
    \(GetSmallest[L_List]\  := \ 
      Module[{e, L1 = L, \ len\  = \ Length[L]}, \n\t\t
        While[\ len\  > \ 0, \ e\  = \ L1[\([1]\)]; \ \n\t\t\t
          If[e\  < \ \(-1\),  (*\ Print[\ "\<Too Divergent\>"]; \ *) \ 
            Return[0]]; \n\t\t\t\tL1\  = \ Rest[L1]; \n\t\t\t\(--len\); \n
          \t\t\t\t]; \n\t\t\t1\t]\)], "Input",
  CellLabel->"In[64]:="],

Cell[BoxData[
    \(CheckDivergence[expr_, \ var_]\  := \ 
      Module[{e = expr, \ L = {}, \ v = var, \ XXT}, \n\t\t
        e\  = \ e\  /. 
            \ {\(-1\) + v -> \ \(-Hold[1 - v]\), \ 
              1 - v -> \ Hold[1 - v]};  (*\ 
          Fighting\ \ Mathematica\ \(again . \)\ *) \n\t\t\n\t
        e\  = \ Expand[e]; \n\t\t\te\  = \ ReleaseHold[e]; \n\t\t
        e\  = \ e\  /. \ 1 - v -> \ XXT; 
        \  (*\ This\ XXT\ trick\ is\ \(necessary\ --\)\ see\ the\ file\ 
            "\<Why I have trouble with Collect.nb\>"\ *) \n\t\t\n\t\t
        L\  = Flatten[
            Append[\ Cases[Collect[e, \ v^_], \ v^_, \ {0, \ Infinity}], \ 
              Cases[Collect[e, \ XXT^_], \ XXT^_, \ {0, \ Infinity}]]]; \n
        \t\t\tL\  = \ L\  //. \ u_^a_\  :> \ Expand[a]; \n\t\t\t
        L\  = \ L\  /. \ \[Epsilon] -> \ 0; \n
        If[GetSmallest[L]\ \  == \ 0, 
          \  (*\ Print[\ "\<Too Divergent\>"]\ *) \ Return[L]]; \n\t\t1\n\t\t]
        \)], "Input",
  CellLabel->"In[65]:="],

Cell[BoxData[
    \(\(Bad[expr_, \ vars_List]\  := \ 
      Module[{}, \n\t\tIf[Length[vars]\  < \ 2, \ Return[True]]; \n\t\t
        If[\ \((\(! FreeQ[expr, \ vars[\([1]\)]]\)\ )\) && \ \ 
            \((\(! FreeQ[expr, \ vars[\([2]\)]]\))\), \ True, \ False]\n\t]
        \ \  (*\ QUICK\ AND\ DIRTY . \ In\ general, \ 
      you' d\ want\ to\ check\ for\ any\ two\ or\ more\ vars\ appearing\ in\ 
        expr, \ not\ just\ the\ first\ and\ second\ *) \)\)], "Input",
  CellLabel->"In[66]:="],

Cell[BoxData[
    \(OnlyHigherThanSquares[expr_, \ var_]\  := \ 
      Module[{e = Expand[expr], L = expr}, \ \n\t\t
        If[Head[e] === Plus, \ e\  = \ List\ @@\ e, \ e\  = \ List[e]]; \n
        \t\t\ \ Plus\ @@
          \((\ \(\((If[CountWithMultiplicity[#, \ var] < 3, \ 0, \ #])\)\ &
                \)\ /@\ \(List\ @@\ e\))\)\n\t\t]\)], "Input",
  CellLabel->"In[67]:="],

Cell[BoxData[
    \(\(HandleIntYYT[expr_]\  := \ 
      Module[{e = expr, \ L}, \n\t\tIf[FreeQ[e, \ IntYYT], \ Return[e]]; \n
        \t\tL\  = \ Cases[e, \ IntYYT[__], \ {0, \ Infinity}]; \t\n\t\t
        L\  = \ Cases[L, \ k^_]; \n\t\tL\  = \ Complement[L, \ {k^2}]; \t\n
        \t\tIf[L\  != \ {}, \ Print[\ "\<Error in HandleIntYYT\>"]; 
          Return[e]]; \t\t\t\n\t\t
        e = e\  //. \ IntYYT[__] -> \ \(-\ Log[\[Mu]^2]\); \n\t\te\ \n\t\t]
        \t\t\t\t\)\)], "Input",
  CellLabel->"In[68]:="],

Cell[BoxData[
    \(GetRepeatedIndices[term_]\  := \ 
      Module[\ {ans, \ elt, \ i, L, \ t = term}, \n\t\t\t\t\t
        L\  = \ GetAtomsInBrackets[t]; \n\t\t\t\ \ L\  = \ \ GetRepeated[L]\t
        \n\t]\)], "Input",
  CellLabel->"In[69]:="],

Cell[BoxData[
    \(GetRepeated[L1_List]\  := \ 
      Module[{ans\  = \ {}, elt, i, \ L = L1, \ len = Length[L]}, \n\t\t
        While[L\  != \ {}, \n\t\t\t\t\t
          If[Count[L, \ L[\([1]\)]]\  > \ 1, \ 
            \(ans\  = \ Append[ans, \ L[\([1]\)]]; \)\n\t\t\t]; \n\t\t\t
          L\  = \ Drop[L, \ {1}]; \n\t\t\t]; \n\t\tUnion[ans]]\)], "Input",
  CellLabel->"In[70]:="],

Cell[BoxData[
    \(GetAtomsInBrackets[term_]\  := \ 
      Module[{ans\  = \ {}, elt, \ i, j, k, len, L, L2, \ t = term}, \n\t\t
        If[FreeQ[t, \ u_[v__]], \ Return[{}]]; \n\t\t
        If[Head[t]\  === \ Times, \ L\  = \ List\ @@\ t, \ L\  = \ {t}]; \n
        \t\tL\  = \ getatoms[L]; \n\t\tL\  = \ GetSymbols[L]; \n\t\t\t
        len\  = \ Length[L]; \n\t\t
        For[i = 1, \ i <= \ len, \ \(i++\), \n\t\t\telt\  = \ L[\([i]\)]; \n
          \t\t\t\t\t (*\t\t
            j = CountWithMultiplicity[t, \ u_[a___, \ elt, \ v___]]; \ \ *) \n
          \t\t\tj = CountWithMultiplicity[t, \ elt]; \ \n\t\t\t\t\t
          If[j\  > 0, \ \(Do[ans\  = \ Append[ans, \ elt], \ {k, \ 1, \ j}]; 
            \)\n\t\t\t]; \n\t]; \n\t\t\tans\ \n\t\t]\)], "Input",
  CellLabel->"In[71]:="],

Cell[BoxData[
    \(getatoms[x_]\  := \ Cases[x, \ _, \ {\(-1\)}]\)], "Input",
  CellLabel->"In[72]:="],

Cell[BoxData[
    \(GetSymbols[x_]\ \  := \ 
      Module[\ {L = x}, \n\t\t
        L\  = \((If[Head[#]\  === \ Symbol, \ #, \ 0]&)\)\ /@\ L; \n\t\t
        L\  = \ Complement[L, \ {0}]\n\t\t]\)], "Input",
  CellLabel->"In[73]:="],

Cell[BoxData[
    \(\( (*\ It' s\ important\ to\ avoid\ recursion\ in\ the\ following\ *) \n
    HoldSumWithDeltasTerm[\ e_, \ SummationIndices_List, \ 
        symbol_:  \[Delta], \ range_: {1, \ 4}]\  := \n\t\ 
      Module[{\ ans, term = e\ , \ L\  = \ SummationIndices, \ len, \ h, i, \ 
          pos, \ sym = symbol, SumIt, \ ran\  = \ range, \ u, \ v, \ w}, \n
        \t\t\t\t\t\tterm\  = \ SimplifyDeltas[term]; \n\t\t\ \ \ \ \ \t\ 
        len\  = \ Length[L]; 
        \n (*\tIf[FreeAllQ[term, \ L], \ Return[term]]; \ NO\ *) \n\t\t
        For[i = len, \ i > 0, \ \(i--\), \ \n\t\t\t\t\t\t
          term\  = \ term\  //. \ sym[u_, \ u_]\  -> \ 1; \n\t\t\t\ \ \ \ \ 
          pos\  = \ \ 
            Position[term, \ \ \ 
              sym[L[\([i]\)], \ u__]\  | \ sym[u__, \ L[\([i]\)]], {0, \ 
                Infinity}, \ 1]; \n
          \t\t\t (*\ 
            BUG\ \(Alert!\)\ If\ fail\ to\ specify\ {0, \ Infinity}\ and\ 
              instead\ use\ just\ Infinity, \ 
            pos\ will\ be\ empty\ on\ things\ like\ 
                \[Delta][\[Alpha], \ \[Beta]] . \ \ 
                  \(Example : \n\t\t\t\t\t\t\ti\)\  = \ 1, \ 
            L[\([i]\)]\  = \ \[Alpha], \ 
            term\  = \ 
              \(\[Delta][\[Alpha], \ \[Beta]] . \ \ pos\ is\ {}\ because\ sym
                  \  = \ \[Delta]\ appears\ as\ the\ head\ of\ \(term!\)\)\ \n
            \t\t\t*) \ \n\t\t\t\t\t\t\n\t\t\t\t\ \ 
          If[\ Length[pos] > \ 0, \ \n\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
            h\  = \(Extract[\ term, \ pos]\)[\([1]\)]; \n
            \t\t\t\t\t\t\ \ \ \ \ \ \ \ \ \ \ \t
            h\  = \ h\ \  //. \ \ 
                sym[L[\([i]\)], \ u__]\  | \ 
                    sym[u__, \ L[\([i]\)]]\ \ \ \  -> \ u; \n
            \t\t\t\t\t\t\t\t\t\t
            term\  = \ 
              term\ \  //. \ 
                v_[u___, \ L[\([i]\)], \ w___]\  -> \ v[u, \ h, \ w]; \n
            \t\t\t\t\t\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ L\  = \ Drop[L, \ {i}]; \n
            \t\t\t\t\t\ \ \ \ \t]; \ \ \  (*\ If\ *) \n\t\t\t\t]; 
        \ \  (*\ For\ *) \n\t\ \ \ \t\t\t\t
        term\  = \ term\  //. \ sym[u_, \ u_]\  -> \ 1; 
        \  (*\ Need\ this\ once\ again, \ 
          to\ replace\ e . g . \ \[Delta][\[Beta], \ \[Beta]]\ by\ 1\n
              \t\t\t\t\tthe\ last\ time\ \(around\ --\)\ when\ i\  = \ 1, \ 
          L[\([i]\)]\  = \ \[Alpha]\ in\ the\ above\ example\ *) \n\t\t\t\t
        L2\  = \ \(Flatten[\ Append[{#}, \ ran]]&\)\ /@\ L; \n\t\t
        If[Length[L2]\  == \ 0, \ 
          Return[term\  //. \ Power[sym[u__], \ v_] -> \ sym[u]]]; \n
        \t\t\t\t\ \t\t
        ans\  = \t\t
          SumIt[term\  //. \ Power[sym[u__], \ v_] -> \ sym[u], \ 
            Sequence\ @@\ L2\ ]; \n\t
        ans\  = \ ans\  //. \ SumIt -> \ Hold[Sum]\n\t]\)\)], "Input",
  CellLabel->"In[74]:="],

Cell[BoxData[
    \(\(HoldSumWithDeltas[\ e_, \ SummationIndices_List, \ 
        symbol_:  \[Delta], \ range_: {1, \ 4}]\  := \n\t\t\ 
      Module[{ans, exp, i, len, \ L\  = \ SummationIndices, L1, \ Lunique}, \n
        \t\t\texp\  = \ Expand[e]; \n\t\t
        If[Head[exp]\  === \ Plus, \ L1\  = \ List\ @@\ exp, \ 
          L1\  = \ List[exp]]; \t\ \n\t\tlen\  = \ Length[L]; \n\t\t\t\t
        Lunique\  = \ \((Unique[#]&)\)\ /@\ L; \n\t\t\ 
        For[i = 1, \ i\  <= \ len, \ \(i++\), \n\t\t\t\t\ \ \ \ \ \ 
          exp\  = \ 
            exp\  //. \ \ 
              \(L[\([i]\)]\)[u___]\  :> \ \(Lunique[\([i]\)]\)\ [u]\n
          \t\t\t\t]; 
        \t\t (*\ Protect\ things\ like\ \n\t\t\t\t\t\t\t\t\t\[Alpha][1, \ 2]\ 
            from\ summation\ when\ \[Alpha]\ is\ a\ summation\ 
            \(index . \)\ \ *) \n\t\t\t\t\t
        ans\  = \ 
          \((HoldSumWithDeltasTerm[#, L, \ symbol, \ range]&)\)\ /@\ L1; \n
        \t\tIf[Length[ans]\  > \ 1, \ ans\  = \ Plus\ @@\ ans, \ 
          ans\  = \ ans[\([1]\)]]; \n (*\t\tans\  = ReleaseHold[ans]; \ *) \n
        \t\tFor[i = 1, \ i\  <= \ len, \ \(i++\), \n\t\t\t\t\ \ \ \ \ \ 
          \(ans\  = \ 
            ans\  //. \ \ 
              \(Lunique[\([i]\)]\)[u___] -> \ \(L[\([i]\)]\)\ [u]; \)\n
          \t\ \ \ \ ]; \ ans\n\t]; \)\)], "Input",
  CellLabel->"In[75]:="],

Cell[BoxData[
    \(DropPositions[L_List, \ pos_List]\  := \ 
      Module[{}, \n\t\t\t\t\ Delete[L, \ \ List\ /@\ pos]\n\t\t]\)], "Input",
  CellLabel->"In[76]:="],

Cell[BoxData[
    \(CasesMultiple[L1_List, \ elts_List]\  := \  (*\ Top\ Level\ Only\ *) \n
      \tModule[{ans = {}, \ L = L1, \ Le\  = \ elts, \ len, \ i}, \n\t\t
        len\  = \ Length[Le]; \n\t\t
        For[i = 1, \ i <= \ len, \ \(i++\), \n\t\t\t\ \ \ \ \ \ \ 
          ans\  = \ Append[ans, \ Cases[L, \ Le[\([i]\)]]]\n\t\t\t]; \n\t\t
        Union[ans]\n\t]\)], "Input",
  CellLabel->"In[77]:="],

Cell[BoxData[
    \(\(TopLevel[expr_]\  := \ If[Head[expr] === Plus, \ 2, \ 1]\t\t\t\)\)], 
  "Input",
  CellLabel->"In[78]:="],

Cell[BoxData[{
    \( (*\ Gets\ lowest - level\ heads\ enclosing\ x_\ *) \n
    \[Beta]GetHeads[expr_, \ x_]\  := \ 
      Module[{e = expr, pos}, \n\t\tpos\  = \ Position[e, \ x]; \n
        \t (*\tIf[pos == {}, \ Return[{}]]; \ *) \n\ 
        pos\  = \ \(\((Drop[#, \ \(-1\)])\)&\)\ /@\ pos; \n
        e = Extract[e, pos]; \nheads\  = \ \(\((Head[#])\)&\)\ /@\ \ e\n\t\t]
        \), 
    \(SetAttributes[GetHeads, \ Listable]\)}], "Input",
  CellLabel->"In[79]:="]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 800}, {0, 572}},
WindowSize->{772, 506},
WindowMargins->{{2, Automatic}, {Automatic, 2}},
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1709, 49, 654, 14, 110, "Input"],
Cell[2366, 65, 615, 12, 110, "Input"],
Cell[2984, 79, 118, 3, 30, "Input"],
Cell[3105, 84, 1080, 20, 270, "Input"],
Cell[4188, 106, 177, 4, 30, "Input"],
Cell[4368, 112, 1645, 29, 530, "Input"],
Cell[6016, 143, 1421, 24, 330, "Input"],
Cell[7440, 169, 560, 10, 190, "Input"],
Cell[8003, 181, 442, 9, 150, "Input"],
Cell[8448, 192, 498, 10, 130, "Input"],
Cell[8949, 204, 205, 4, 70, "Input"],
Cell[9157, 210, 434, 8, 130, "Input"],
Cell[9594, 220, 662, 11, 250, "Input"],
Cell[10259, 233, 91, 2, 30, "Input"],
Cell[10353, 237, 235, 5, 50, "Input"],
Cell[10591, 244, 423, 8, 150, "Input"],
Cell[11017, 254, 511, 10, 150, "Input"],
Cell[11531, 266, 504, 10, 130, "Input"],
Cell[12038, 278, 411, 8, 150, "Input"],
Cell[12452, 288, 516, 9, 130, "Input"],
Cell[12971, 299, 3496, 62, 830, "Input"],
Cell[16470, 363, 2959, 53, 650, "Input"],
Cell[19432, 418, 1459, 28, 350, "Input"],
Cell[20894, 448, 1547, 28, 310, "Input"],
Cell[22444, 478, 446, 8, 190, "Input"],
Cell[22893, 488, 1805, 29, 610, "Input"],
Cell[24701, 519, 1215, 24, 250, "Input"],
Cell[25919, 545, 441, 9, 110, "Input"],
Cell[26363, 556, 471, 9, 110, "Input"],
Cell[26837, 567, 496, 11, 130, "Input"],
Cell[27336, 580, 175, 4, 30, "Input"],
Cell[27514, 586, 691, 13, 230, "Input"],
Cell[28208, 601, 470, 9, 170, "Input"],
Cell[28681, 612, 231, 6, 70, "Input"],
Cell[28915, 620, 151, 3, 30, "Input"],
Cell[29069, 625, 4364, 72, 890, "Input"],
Cell[33436, 699, 239, 5, 90, "Input"],
Cell[33678, 706, 200, 4, 70, "Input"],
Cell[33881, 712, 2608, 50, 630, "Input"],
Cell[36492, 764, 482, 8, 150, "Input"],
Cell[36977, 774, 1689, 28, 590, "Input"],
Cell[38669, 804, 98, 2, 30, "Input"],
Cell[38770, 808, 2721, 49, 690, "Input"],
Cell[41494, 859, 911, 15, 190, "Input"],
Cell[42408, 876, 1499, 28, 230, "Input"],
Cell[43910, 906, 827, 15, 130, "Input"],
Cell[44740, 923, 899, 17, 270, "Input"],
Cell[45642, 942, 3336, 59, 838, "Input"],
Cell[48981, 1003, 1314, 25, 390, "Input"],
Cell[50298, 1030, 1016, 17, 290, "Input"],
Cell[51317, 1049, 948, 18, 290, "Input"],
Cell[52268, 1069, 1888, 33, 490, "Input"],
Cell[54159, 1104, 1498, 30, 330, "Input"],
Cell[55660, 1136, 5259, 92, 1401, "Input"],
Cell[60922, 1230, 385, 7, 150, "Input"],
Cell[61310, 1239, 1004, 19, 310, "Input"],
Cell[62317, 1260, 494, 8, 130, "Input"],
Cell[62814, 1270, 380, 7, 90, "Input"],
Cell[63197, 1279, 510, 9, 190, "Input"],
Cell[63710, 1290, 245, 5, 90, "Input"],
Cell[63958, 1297, 383, 7, 150, "Input"],
Cell[64344, 1306, 787, 13, 310, "Input"],
Cell[65134, 1321, 103, 2, 30, "Input"],
Cell[65240, 1325, 230, 5, 90, "Input"],
Cell[65473, 1332, 2848, 52, 610, "Input"],
Cell[68324, 1386, 1355, 25, 350, "Input"],
Cell[69682, 1413, 164, 3, 70, "Input"],
Cell[69849, 1418, 406, 7, 170, "Input"],
Cell[70258, 1427, 127, 3, 30, "Input"],
Cell[70388, 1432, 469, 9, 190, "Input"]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

